#include "Header.hh"
  
void InitPlant()
{
  SetUpTissueLayout();
  NumberCells();
  FindCellCorners();
  FindCellSizes();
  SetUpMembraneTypes();
  SetUpPrePattern();
  InitConditions();
  UpdateMembraneProteinsAndFluxes(0);
}

void WritePlantToFile(int time)
{
  FILE *f;
  char fname[800];
  sprintf(fname,"%s/DumpPlantStateAt%.10d",dirname,time);
  f=fopen(fname,"wb");

  //tissue architecture and cell numbering
  fwrite(Tc,sizeof(Tc),1,f);
  fwrite(Tct,sizeof(Tct),1,f);
  fwrite(Tmt,sizeof(Tmt),1,f);
  fwrite(Cell,sizeof(Cell),1,f);
  //location of transporters
  fwrite(PinPrePat,sizeof(PinPrePat),1,f);
  fwrite(Lax3PrePat,sizeof(Lax3PrePat),1,f);
  fwrite(Lax2PrePat,sizeof(Lax2PrePat),1,f);
  fwrite(Aux1PrePat,sizeof(Aux1PrePat),1,f);
  //auxin levels
  fwrite(Caux1,sizeof(Caux1),1,f);
  fwrite(MeanCells,sizeof(MeanCells),1,f);
  //gene expression levels
  fwrite(ARF,sizeof(ARF),1,f);
  fwrite(GG,sizeof(GG),1,f);
  fwrite(AA,sizeof(AA),1,f);
  fwrite(BB,sizeof(BB),1,f);
  fwrite(CC,sizeof(CC),1,f);
  fwrite(DD,sizeof(DD),1,f);
  fwrite(Plt,sizeof(Plt),1,f);
  fwrite(Yuc3,sizeof(Yuc3),1,f);
  fwrite(Gh33,sizeof(Gh33),1,f);
  fwrite(Pin,sizeof(Pin),1,f);
  fwrite(Plts,sizeof(Plts),1,f);
  fwrite(Lax3,sizeof(Lax3),1,f);
  fwrite(Lax2,sizeof(Lax2),1,f);
  fwrite(Aux1,sizeof(Aux1),1,f);
  //miscelleanous
  fwrite(TimePrevGrowth,sizeof(TimePrevGrowth),1,f);
  fwrite(DivRate,sizeof(DivRate),1,f);
  fwrite(GrowthRate,sizeof(GrowthRate),1,f);
  fwrite(Diff,sizeof(Diff),1,f);
  fwrite(CellZonationState,sizeof(CellZonationState),1,f);
  //nr of cells
  fwrite(&actcells,sizeof(actcells),1,f);
  fclose(f);
}

void InitPlantFromFile(int startt)
//void InitPlantFromFile(char *fname)
{
  FILE *f;
  char fname2[800];

  sprintf(fname2,"%s/DumpPlantStateAt%.10d",dirname,startt);
  //sprintf(fname2,"%s/%s",dirname,fname);
  f=fopen(fname2,"rb");
  //tissue architecture and cell numbering
  fread(Tc,sizeof(Tc),1,f);
  fread(Tct,sizeof(Tct),1,f);
  fread(Tmt,sizeof(Tmt),1,f);
  fread(Cell,sizeof(Cell),1,f);
  //location of transporters
  fread(PinPrePat,sizeof(PinPrePat),1,f);
  fread(Lax3PrePat,sizeof(Lax3PrePat),1,f);
  fread(Lax2PrePat,sizeof(Lax2PrePat),1,f);
  fread(Aux1PrePat,sizeof(Aux1PrePat),1,f);
  //auxin levels
  fread(Caux1,sizeof(Caux1),1,f);
  fread(MeanCells,sizeof(MeanCells),1,f);
  //gene expression levels
  fread(ARF,sizeof(ARF),1,f);
  fread(GG,sizeof(GG),1,f);
  fread(AA,sizeof(AA),1,f);
  fread(BB,sizeof(BB),1,f);
  fread(CC,sizeof(CC),1,f);
  fread(DD,sizeof(DD),1,f);
  fread(Plt,sizeof(Plt),1,f);
  fread(Yuc3,sizeof(Yuc3),1,f);
  fread(Gh33,sizeof(Gh33),1,f);
  fread(Pin,sizeof(Pin),1,f);
  fread(Plts,sizeof(Plts),1,f);
  fread(Lax3,sizeof(Lax3),1,f);
  fread(Lax2,sizeof(Lax2),1,f);
  fread(Aux1,sizeof(Aux1),1,f);
  //miscelleanous
  fread(TimePrevGrowth,sizeof(TimePrevGrowth),1,f);
  fread(DivRate,sizeof(DivRate),1,f);
  fread(GrowthRate,sizeof(GrowthRate),1,f);
  fread(Diff,sizeof(Diff),1,f);
  fread(CellZonationState,sizeof(CellZonationState),1,f);
  //nr of cells
  fread(&actcells,sizeof(actcells),1,f);
  fclose(f);
  FindCellCorners();
  FindCellSizes();
  //SetUpPrePattern();//!
  //InitConditions();//!
  UpdateMembraneProteinsAndFluxes(0);
}


void SetUpTissueLayout()
{
  int i,j,k,l,ii;
  /*****************************************************************/  
  
  //in Tc[n][m] 
  //0 = wall
  //1 = cell
  //2 = membrane
  //3 = non-tissue
  
  //in Tct[n][m] 
  //0 = no cell
  //1 = lateral root cap 1
  //2 = lateral root cap 2
  //3 = lateral root cap 3
  //4 = lateral root cap 4
  //5 = epi
  //6 = cort
  //7 = endo
  //8 = peri
  //9 = vasc 1
  //10 = vasc 2
  //11 = vasc 3
  //12 = QC
  //13 = columella 1
  //14 = columella 2
  //15 = columella 3
  //16 = columella 4
  //17 = columella 5
  //18 = columella initials

  /*****************************************************************/
  for(j=0;j<m;j++)
    for(i=0;i<15;i++)
      istart[j][i]=0;


  /******************************************************************/

  //0: Initially put all to non-tissue, no cell
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	Tc[i][j]=3;
	Tct[i][j]=0;
      }

  //1: Now, set up where there is plant tissue and where not
  //Step a: generate straight part of tissue without root cap
  for(i=xm;i>=totsizecap ;i--)
    for(j=yLRC1;j<yEND;j++)
      {
	ii=n-1-i;
	Tc[i][j]=1;//make all cell, walls and membranes done later
	Tc[ii][j]=1;
	Tct[i][j]=5;//make all epidermal, other types done later
	Tct[ii][j]=5;
      }
  //Step b: generate curved part of tissue with root cap
  //for this use parabola describing the outer boundary of
  //the outermost root cap layer LRC1
  for(i=xm;i>=0;i--)
    for(j=0;j<yLRC1;j++)
      {
 	ii=n-1-i;
	if(j>=(int)(aLRC1*abs(pow((i-xm),powLRC1))+bLRC1))
	  {
	    Tc[i][j]=1;//make all cell, walls and membranes done later
	    Tc[ii][j]=1;
	    Tct[i][j]=1;//make all lateral root cap type, other types done later
	    Tct[ii][j]=1;
	  }
      }
  /************************************************************/

  //2: Next, setup where is cell wall and where are different cell types
  int temp;
  double iii,jjj;
  int I;
  //Step a: in straight part of tissue without root cap
  //vertical walls & celltypes
  for(j=yLRC1;j<yEND;j++)
    {
      i=totsizecap;
      ii=n-1-i;
      Tc[i][j]=0;
      Tc[ii][j]=0;

      i=xm;
      Tc[i][j]=0;
      for(k=0;k<nrepi+nrcort+nrendo+nrperi+nrvasc;k++)
	{
	  if(k<nrvasc)
	    {
	      //subdivide later 9-10-11
	      for(I=i;I>=i-(sizevasc+1);I--)
		{
		  Tct[I][j]=9;
		  Tct[n-1-I][j]=9;
		}
	      i-=sizevasc+1;
	    }
	  else if(k<nrvasc+nrperi)
	    {
	      for(I=i;I>=i-(sizeperi+1);I--)
		{
		  Tct[I][j]=8;
		  Tct[n-1-I][j]=8;
		}
	      i-=sizeperi+1;
	    }
	  else if(k<nrvasc+nrperi+nrendo)
	    {
	      for(I=i;I>=i-(sizeendo+1);I--)
		{
		  Tct[I][j]=7;
		  Tct[n-1-I][j]=7;
		}
	      i-=sizeendo+1;
	    }
	  else if(k<nrvasc+nrperi+nrendo+nrcort)
	    {
	      for(I=i;I>=i-(sizecort+1);I--)
		{
		  Tct[I][j]=6;
		  Tct[n-1-I][j]=6;
		}
	      i-=sizecort+1;
	    }
	  else if(k<nrvasc+nrperi+nrendo+nrcort+nrepi)
	    i-=sizeepi+1;
	  
	  Tc[i][j]=0;
	  ii=n-1-i;
	  Tc[ii][j]=0;
	}
    }
  //Step b: in curved part of tissue with root cap
  //semi-vertical lines (parabola) & celltypes
  //first walls
  for(i=xm;i>=0;i--)
    for(j=0;j<yLRC1;j++)
      {
	ii=n-1-i;
	if(Tc[i][j]!=3)
	  {
	    if(j==(int)(aVASC*abs(pow((i-xm),powVASC))+bVASC) ||
	       i==n-1-(int)(pow(((j-bVASC)/aVASC),(1.0/powVASC))+xm))//MAYBE NOT TILL END
	      {
		Tc[i][j]=0;
		Tc[ii][j]=0;
	      }
	    else if(j==(int)(aPERI*abs(pow((i-xm),powPERI))+bPERI) ||
		    i==n-1-(int)(pow(((j-bPERI)/aPERI),(1.0/powPERI))+xm))
	      {
		Tc[i][j]=0;
		Tc[ii][j]=0;
	      }
	    else if(j==(int)(aENDO*abs(pow((i-xm),powENDO))+bENDO) ||
		    i==n-1-(int)(pow(((j-bENDO)/aENDO),(1.0/powENDO))+xm))
	      {
		if(j>(int)(arad4*i+brad4))
		  {
		    Tc[i][j]=0;
		    Tc[ii][j]=0;
		  }
	      }
	    else if(j==(int)(aCORT*abs(pow((i-xm),powCORT))+bCORT) ||
		    i==n-1-(int)(pow(((j-bCORT)/aCORT),(1.0/powCORT))+xm))
	      {
		Tc[i][j]=0;
		Tc[ii][j]=0;
	      }
	    else if(j==(int)(aEPI*abs(pow((i-xm),powEPI))+bEPI)||
		    i==n-1-(int)(pow(((j-bEPI)/aEPI),(1.0/powEPI))+xm))
	      {
		Tc[i][j]=0;
		Tc[ii][j]=0;
	      }
	    else if(j<yLRC4 && (j==(int)(aLRC4*abs(pow((i-xm),powLRC4))+bLRC4) ||
				i==n-1-(int)(pow(((j-bLRC4)/aLRC4),(1.0/powLRC4))+xm)) )
	      {
		Tc[i][j]=0;
		Tc[ii][j]=0;
	      }
	    else if(j<yLRC3 && (j==(int)(aLRC3*abs(pow((i-xm),powLRC3))+bLRC3) ||
				i==n-1-(int)(pow(((j-bLRC3)/aLRC3),(1.0/powLRC3))+xm)) )
	      {
		Tc[i][j]=0;
		Tc[ii][j]=0;
	      }
	    else if(j<yLRC2 && (j==(int)(aLRC2*abs(pow((i-xm),powLRC2))+bLRC2) ||
			   i==n-1-(int)(pow(((j-bLRC2)/aLRC2),(1.0/powLRC2))+xm)) )
	      {
		Tc[i][j]=0;
		Tc[ii][j]=0;
	      }
	    else if(j==(int)(aLRC1*abs(pow((i-xm),powLRC1))+bLRC1) ||
		    i==n-1-(int)(pow(((j-bLRC1)/aLRC1),(1.0/powLRC1))+xm))
	      {
		Tc[i][j]=0;
		Tc[ii][j]=0;
	      }
	  }
      }
  //then celltypes
  for(i=xm;i>=0;i--)
    for(j=0;j<yLRC1;j++)
      {
	if(Tc[i][j]!=3)
	  {
	    ii=n-1-i;
	    //parabola like curves

	    //met de power 5 gaat er nog wat mis met celltypes
	    //opgelost met i>60, niet echt mooi
	    	
	   
	    //subdivide later 9-10-11
	    if(i>60 && j>=(int)(aVASC*abs(pow((i-xm),powVASC))+bVASC))
	      {
		Tct[i][j]=9;
		Tct[ii][j]=9;
	      }
	    else if(i>60 && j>=(int)(aPERI*abs(pow((i-xm),powPERI))+bPERI))
	      {
		Tct[i][j]=8;
		Tct[ii][j]=8;
	      }
	    else if(j>=(int)(aENDO*abs(pow((i-xm),powENDO))+bENDO))
	      {
		Tct[i][j]=7;
		Tct[ii][j]=7;
	      }
	    else if(j>=(int)(aCORT*abs(pow((i-xm),powCORT))+bCORT))
	      {
		Tct[i][j]=6;
		Tct[ii][j]=6;
	      }
	    else if(j>=(int)(aEPI*abs(pow((i-xm),powEPI))+bEPI) )
	      {
		Tct[i][j]=5;
		Tct[ii][j]=5;
	      }
	    else if(j<(int)(arad2*i+brad2) && j>=(int)(aLRC4*abs(pow((i-xm),powLRC4))+bLRC4))
	      {
		Tct[i][j]=4;
		Tct[ii][j]=4;
	      }
	    else if(j<yLRC3 && j>=(int)(aLRC3*abs(pow((i-xm),powLRC3))+bLRC3))
	      {
		Tct[i][j]=3;
		Tct[ii][j]=3;
	      }
	    else if(j<yLRC2 && j>=(int)(aLRC2*abs(pow((i-xm),powLRC2))+bLRC2))
	      {
		Tct[i][j]=2;
		Tct[ii][j]=2;
	      }
	  }
      }
  //some more celltypes
  //QC and columella
   for(i=xm;i>=0;i--)
     for(j=0;j<yLRC1;j++)
       {
	 ii=n-1-i;
	 if(Tc[i][j]!=3)
	   {
	     //different collumella layers
	     if(Tct[i][j]==1 && j<(int)(arad3*i+brad3))//LRC1,2,3,4,of epi
	       {
		 Tct[i][j]=13;
		 Tct[ii][j]=13;
	       }
	     else if(Tct[i][j]==2 && j<(int)(arad3*i+brad3))//LRC1,2,3,4,of epi
	       {
		 Tct[i][j]=14;
		 Tct[ii][j]=14;
	       }
	     else if(Tct[i][j]==3 && j<(int)(arad3*i+brad3))//LRC1,2,3,4,of epi
	       {
		 Tct[i][j]=15;
		 Tct[ii][j]=15;
	       }
	     else if(Tct[i][j]==4 && j<(int)(arad3*i+brad3))//LRC1,2,3,4,of epi
	       {
		 Tct[i][j]=16;
		 Tct[ii][j]=16;
	       }
	     else if(Tct[i][j]==5 && j<(int)(arad3*i+brad3))//LRC1,2,3,4,of epi
	       {
		 Tct[i][j]=17;
		 Tct[ii][j]=17;

		 //zou evt opgesplitst in laag 15 en laag 16 moeten worden
	       }
	    
	     //QC
	     if((Tct[i][j]==6 || Tct[i][j]==7) && j<(int)(arad4*i+brad4))//cortex -> QC
	       {
		 Tct[i][j]=12;
		 Tct[ii][j]=12;
	       }
	   }
       }
   //above yR vertical lines
   for(j=yR;j<=yLRC1;j++)
    {
      i=xm;
      Tc[i][j]=0;

     
      for(k=0;k<nrvasc-1;k++)
	{
	  //if(k<nrvasc)
	  if(j>yR+8*HMZ+8)
	    i-=sizevasc+1;
	  else if(j>yR+4*HMZ+4)
	    i-=sizevasc;
	  else
	    i-=sizevasc-1;

	 
	  if(k<nrvasc-2 || j>yR)
	    {
	      Tc[i][j]=0;
	      ii=n-1-i;
	      Tc[ii][j]=0;
	    }
	}
    }

  /**************************************************************/
  
  //first determine outlines of different cell files
  //only after that put in horizontal walls
  int wallcounter;
  for(j=yR;j<yEND;j++)
    {
      wallcounter=0;
      
      for(i=0;i<n;i++)
	{
	  if(Tc[i][j]==0)//point of wall found
	    {
	      if((i<n/2 && Tct[i+2][j]>4) || (i>=n/2 && Tct[i-2][j]>4))//no LRC etc
		{
		  istart[j][wallcounter]=i;
		  while(Tc[i][j]==0)//skip further points of same wall
		    i++;
		  wallcounter++;
		}
	    }
	}
    }
  //we want to continue the straight walls in the upper part all through the grid
  //otherwise there are no "guide rulers" going upward when the cells grow
  for(j=yEND;j<m;j++)
    {
      for(i=0;i<15;i++)
	istart[j][i]=istart[j-1][i];
    }

  /******************************************************************/


  //straight part of tissue without root cap
  //horizontal walls, different cellsizes (heights) in different zones
  for(i=xm;i>=totsizecap;i--)
    {
      ii=n-1-i;

      j=yLRC1;
      Tc[i][j]=0;
      Tc[ii][j]=0;

      
      for(k=1;k<=12;k++)//CHANGED!
	{
	  if(k==1)
	    j+=20;
	  else if(k==2)
	    j+=21;//23;
	  else if(k==3)
	    j+=23;//29;
	  else if(k==4)
	    j+=27;//41;
	  else if(k==5)
	    j+=34;
	  else if(k==6)
	    j+=44;
	  else if(k==7)
	    j+=60;
	  else if(k==8)
	    j+=82;
	  else
	    j+=110;

	   if(j<yEND)
	    {
	      Tc[i][j]=0;
	      Tc[ii][j]=0;
	    }
	}

      /*
      for(k=1;k<=9;k++)//EZ cells in straight part
	{
	  j+=HEZ+1;
	  if(j<yEND)
	    {
	      Tc[i][j]=0;
	      Tc[ii][j]=0;
	    }
	}
      for(k=1;k<=5;k++)//DZ cells in straight part
	{
	  j+=HDZ+1;
	  if(j<yEND)
	    {
	      Tc[i][j]=0;
	      Tc[ii][j]=0;
	    }
	}
      */

      j=yEND-1;
      Tc[i][j]=0;
      Tc[ii][j]=0;
    } 


  //in curved part of tissue with root cap
  //horizontal and radial lines
  //above yR horizontal lines
  for(i=xm;i>=0;i--)
    {
      ii=n-1-i;
      j=yR;
     

      //first horizontal line 
      if(Tc[i][j]!=3 )//&& j>(int)(aLRC3*abs(pow((i-xm),powLRC3))+bLRC3))
	{
	  Tc[i][j]=0;
	  Tc[ii][j]=0;
	}
      
      //horizontal walls for MZ cells between yR and yMZEZ inside epi parabola
      for(k=1;k<=16;k++ )
	{
	  j+=HMZ+1;
	  if(Tc[i][j]!=3 && j>(int)(aEPI*abs(pow((i-xm),powEPI))+bEPI))
	    {
	      Tc[i][j]=0;
	      Tc[ii][j]=0;
	    }
	}
      
      j+=HMZ+3;//NEW!!
      if(Tc[i][j]!=3 && j>(int)(aEPI*abs(pow((i-xm),powEPI))+bEPI))
	{
	  Tc[i][j]=0;
	  Tc[ii][j]=0;
	}
      j+=HMZ+6;//NEW!!
      if(Tc[i][j]!=3 && j>(int)(aEPI*abs(pow((i-xm),powEPI))+bEPI))
	{
	  Tc[i][j]=0;
	  Tc[ii][j]=0;
	}

      //horizontal walls for LRC cells between yR and yLRC1 outside epi parabola 
      j=yR;
      for(k=1;k<=6;k++ )
	{
	  if(k<=2)
	    j+=2*HMZ+2;
	  else if(k<=4)
	    j+=3*HMZ+3;
	  else
	    j+=3.5*HMZ+3;
	  if(Tc[i][j]!=3 && j<(int)(aEPI*abs(pow((i-xm),powEPI))+bEPI))
	    {
	      Tc[i][j]=0;
	      Tc[ii][j]=0;
	    }
	}
      
      //horizontal walls for final cells LRC1
      j=yLRC1-1;
      if(Tc[i][j]!=3 && j<=(int)(aEPI*abs(pow((i-xm),powEPI))+bEPI))
	{
	  Tc[i][j]=0;
	  Tc[ii][j]=0;
	}
    }
  
  //below yR, so from yQC onward, inside endo parabola
  //horizontal
  for(i=xm;i>=0;i--)
    {
      ii=n-1-i;
      j=yQC;

      for(k=1;k<=4;k++ )
	{
	  j+=HMZ+1;
	  if(Tc[i][j]!=3 && j>(int)(aVASC*abs(pow((i-xm),powVASC))+bVASC))
	    {
	      Tc[i][j]=0;
	      Tc[ii][j]=0;
	    }
	}
    }
  //below yR, so from yQC onward, inside endo parabola
  //vertical
  for(j=yQC;j<yR;j++)
    {
      i=xm;
      
      for(k=0;k<nrvasc-2;k++)
	{
	  //if(k<nrvasc)
	  if(j>yQC+2*HMZ+2)
	    i-=sizevasc+1;
	  else
	    i-=sizevasc-1;

	  if(k<nrvasc-3 || j>yQC+1*HMZ+1)
	    {
	      Tc[i][j]=0;
	      ii=n-1-i;
	      Tc[ii][j]=0;
	    }
	}
    }
  //below yR, inside LRC1 parabola
  //radial lines and vertical line
  for(i=xm;i>=0;i--)
    for(j=0;j<yR;j++)
      {
	ii=n-1-i;

	if(j>=(int)(aLRC1*abs(pow((i-xm),powLRC1))+bLRC1))
	  {
	    if(j<=(int)(aVASC*abs(pow((i-xm),powVASC))+bVASC) &&
	       (j==(int)(arad1*i+brad1) || i==(int)((j-brad1)/arad1)))
	      {
		Tc[i][j]=0;
		Tc[ii][j]=0;
	      }
	    else if(j<=(int)(aVASC*abs(pow((i-xm),powVASC))+bVASC) &&
		     (j==(int)(arad2*i+brad2) || i==(int)((j-brad2)/arad2)))
	      {
		Tc[i][j]=0;
		Tc[ii][j]=0;
	      }
	    else if(j<=(int)(aVASC*abs(pow((i-xm),powVASC))+bVASC) &&
		    (j==(int)(arad3*i+brad3) || i==(int)((j-brad3)/arad3)))
	      {
		Tc[i][j]=0;
		Tc[ii][j]=0;
	      }
	    else if(j<=(int)(aVASC*abs(pow((i-xm),powVASC))+bVASC) &&
		    (j==(int)(arad4*i+brad4) || i==(int)((j-brad4)/arad4)))
	      {
		Tc[i][j]=0;
		Tc[ii][j]=0;
	      }
	    else if(i==xm)
	      {
		Tc[i][j]=0;
		Tc[ii][j]=0;
	      }
	  }
      }
  /************************************************************/
  //next setup membranes based on cell walls
  for(i=0;i<n;i++)
    for(j=0;j<m-bufsize;j++)
      {
	if(Tc[i][j]==1)//cell
	  {
	    if(Tc[i+1][j]==0 || Tc[i-1][j]==0 || Tc[i][j+1]==0 || Tc[i][j-1]==0)//nb is wall
	      Tc[i][j]=2; //membrane
	  }
      } 
  
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)//cell wall
	  Tct[i][j]=20;//to get different colour there
      }
 
  /***********************************************************/

  //still need to subdivide vasc into 9 -10 -11
  int wallcounter2;
  for(j=0;j<m;j++)
    {
      wallcounter2=0;
      for(i=xm;i>=0;i--)
	{
	  ii=n-1-i;
	  while(Tc[i][j]==0)
	    {
	      i--;
	      ii=n-1-i;
	    }
	  if(Tct[i][j]<9)
	    {
	      i=-100;//no longer vasculature so end routine
	    }
	  else
	    {
	      while(Tct[i][j]==9)
		{
		  if(wallcounter2==0)//first cell files from middle out
		    {
		      Tct[i][j]=11;
		      Tct[ii][j]=11;
		    }
		  else if(wallcounter2==1)//second cell files from middle out
		    {
		      Tct[i][j]=10;
		      Tct[ii][j]=10;
		    }
		  else if(wallcounter2==2)
		    ;
		  i--;
		  ii=n-1-i;
		}
	      wallcounter2++;
	    }
	}
    }

  /* 
  for(j=yR;j<m;j++)
    {
      printf("%i\t",j);
      for(i=0;i<15;i++)
	printf("%i\t",istart[j][i]);
      printf("\n");
    }
  exit(1);  
  */
}

void SetUpMembraneTypes()
{
  int i,ii,j,jj,q,iii,jjj;

  //bepaal membrane orientatie: 1: top, 2: bottom, 3: in, 4:out
  //op basis van waar is membraan Tc[i][j]==2, wat voor celltype is het
  //Tct[i][j]=??? en wat is de orientatie van de membraan en dus nearby
  //cell wall Twv[i][j]=???

  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)
	  Tmt[i][j]=10;//dummy orientation if a membrane
	else
	  Tmt[i][j]=0;//no orientation if not a membrane
      }
  
  
  //straight part of root + part of rootcap that is more or less straight
  for(i=xm;i>=0;i--)
    for(j=yR;j<=yEND;j++)
      {
	ii=n-1-i;
	if(Tmt[i][j]==10)//there is an unidentified membrane
	  {
	    if(j==CellMaxJ[Cell[i][j]])//top
	      {
		Tmt[i][j]=1;
		Tmt[ii][j]=1;
	      }
	    else if(j==CellMinJ[Cell[i][j]])//bottom
	      {
		Tmt[i][j]=2;
		Tmt[ii][j]=2;
	      }
	    else if(Tc[i+1][j]==0)//right
	      {
		Tmt[i][j]=3;
		Tmt[ii][j]=3;
	      }
	    else if(Tc[i-1][j]==0)//left
	      {
		Tmt[i][j]=4;
		Tmt[ii][j]=4;
	      }
	  }
      }
  //end of straight part of root + part of rootcap that is more or less straight
  
  
  //curved part of root, only vasculature
  for(i=xm;i>=0;i--)
    for(j=yQC;j<=yR;j++)
      {
	ii=n-1-i;
	if(Tmt[i][j]==10)//there is an unidentified membrane
	  {
	    if(Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11)
	      {
		if(j==CellMaxJ[Cell[i][j]])//top
		  {
		    Tmt[i][j]=1;
		    Tmt[ii][j]=1;
		  }
		else if(j==CellMinJ[Cell[i][j]])//bottom
		  {
		    Tmt[i][j]=2;
		    Tmt[ii][j]=2;
		  }
		else if(Tc[i+1][j]==0)//right
		  {
		    Tmt[i][j]=3;
		    Tmt[ii][j]=3;
		  }
		else if(Tc[i-1][j]==0)//left
		  {
		    Tmt[i][j]=4;
		    Tmt[ii][j]=4;
		  }
	      }
	  }
      }
  //end of curved part of root, only vasculature

  //curved part of root
  //inward and outward oriented membranes
  for(i=xm;i>=0;i--)
    for(j=yzero;j<=yR;j++)
      {
	ii=n-1-i;
	if(Tmt[i][j]==10)//there is an unidentified membrane
	  {
	    //outward pointing membranes
	    if(Tc[i-2][j-2]==3)//outmost cell membrane
	      {
		Tmt[i][j]=4;
		Tmt[ii][j]=4;
	      }
	    else if((i-1)==n-1-(int)(pow(((j-bLRC2)/aLRC2),(1.0/powLRC2))+xm) ||
		    (j-1)==(int)(aLRC2*abs(pow((i-xm),powLRC2))+bLRC2))
	      {
		Tmt[i][j]=4;
		Tmt[ii][j]=4;
	      }
	    else if((i-1)==n-1-(int)(pow(((j-bLRC3)/aLRC3),(1.0/powLRC3))+xm) ||
		    (j-1)==(int)(aLRC3*abs(pow((i-xm),powLRC3))+bLRC3))
	      {
		Tmt[i][j]=4;
		Tmt[ii][j]=4;
	      }
	    else if(j<yLRC4 && 
		    ((i-1)==n-1-(int)(pow(((j-bLRC4)/aLRC4),(1.0/powLRC4))+xm) ||
		     (j-1)==(int)(aLRC4*abs(pow((i-xm),powLRC4))+bLRC4)))
	      {
		Tmt[i][j]=4;
		Tmt[ii][j]=4;
	      }
	    else if((i-1)==n-1-(int)(pow(((j-bEPI)/aEPI),(1.0/powEPI))+xm) ||
		    (j-1)==(int)(aEPI*abs(pow((i-xm),powEPI))+bEPI))
	      {
		Tmt[i][j]=4;
		Tmt[ii][j]=4;
	      }
	    else if((i-1)==n-1-(int)(pow(((j-bCORT)/aCORT),(1.0/powCORT))+xm) ||
		    (j-1)==(int)(aCORT*abs(pow((i-xm),powCORT))+bCORT))
	      {
		Tmt[i][j]=4;
		Tmt[ii][j]=4;
	      }
	    else if((i-1)==n-1-(int)(pow(((j-bENDO)/aENDO),(1.0/powENDO))+xm) ||
		    (j-1)==(int)(aENDO*abs(pow((i-xm),powENDO))+bENDO))
	      {
		if(j>(int)(arad4*i+brad4))
		  {
		    Tmt[i][j]=4;
		    Tmt[ii][j]=4;
		  }
	      }
	    else if((i-1)==n-1-(int)(pow(((j-bPERI)/aPERI),(1.0/powPERI))+xm) ||
		    (j-1)==(int)(aPERI*abs(pow((i-xm),powPERI))+bPERI))
	      {
		Tmt[i][j]=4;
		Tmt[ii][j]=4;
	      }
	    else if((i-1)==n-1-(int)(pow(((j-bVASC)/aVASC),(1.0/powVASC))+xm) ||
		    (j-1)==(int)(aVASC*abs(pow((i-xm),powVASC))+bVASC))
	      {
		Tmt[i][j]=4;
		Tmt[ii][j]=4;
	      }
	    
	    
	    //inward pointing membranes
	    if((i+1)==n-1-(int)(pow(((j-bLRC2)/aLRC2),(1.0/powLRC2))+xm) ||
	       (j+1)==(int)(aLRC2*abs(pow((i-xm),powLRC2))+bLRC2))
	      {
		Tmt[i][j]=3;
		Tmt[ii][j]=3;
	      }
	    else if((i+1)==n-1-(int)(pow(((j-bLRC3)/aLRC3),(1.0/powLRC3))+xm) ||
	       (j+1)==(int)(aLRC3*abs(pow((i-xm),powLRC3))+bLRC3))
	      {
		Tmt[i][j]=3;
		Tmt[ii][j]=3;
	      }
	    else if(j<yLRC4 && 
		    ((i+1)==n-1-(int)(pow(((j-bLRC4)/aLRC4),(1.0/powLRC4))+xm) ||
		     (j+1)==(int)(aLRC4*abs(pow((i-xm),powLRC4))+bLRC4)))
	      {
		Tmt[i][j]=3;
		Tmt[ii][j]=3;
	      }
	    else if((i+1)==n-1-(int)(pow(((j-bEPI)/aEPI),(1.0/powEPI))+xm) ||
	       (j+1)==(int)(aEPI*abs(pow((i-xm),powEPI))+bEPI))
	      {
		Tmt[i][j]=3;
		Tmt[ii][j]=3;
	      }
	    else if((i+1)==n-1-(int)(pow(((j-bCORT)/aCORT),(1.0/powCORT))+xm) ||
	       (j+1)==(int)(aCORT*abs(pow((i-xm),powCORT))+bCORT))
	      {
		Tmt[i][j]=3;
		Tmt[ii][j]=3;
	      }
	    else if((i+1)==n-1-(int)(pow(((j-bENDO)/aENDO),(1.0/powENDO))+xm) ||
	       (j+1)==(int)(aENDO*abs(pow((i-xm),powENDO))+bENDO))
	      {
		if(j>(int)(arad4*i+brad4))
		  {
		    Tmt[i][j]=3;
		    Tmt[ii][j]=3;
		  }
	      }
	    else if((i+1)==n-1-(int)(pow(((j-bPERI)/aPERI),(1.0/powPERI))+xm) ||
	       (j+1)==(int)(aPERI*abs(pow((i-xm),powPERI))+bPERI))
	      {
		Tmt[i][j]=3;
		Tmt[ii][j]=3;
	      }
	    else if((i+1)==n-1-(int)(pow(((j-bVASC)/aVASC),(1.0/powVASC))+xm) ||
	       (j+1)==(int)(aVASC*abs(pow((i-xm),powVASC))+bVASC))
	      {
		Tmt[i][j]=3;
		Tmt[ii][j]=3;
	      }	     
	  }
      }
  //repair missing parts
  int nb_3;
  int nb_4;
  int temp[n][m];
  int th=2;
  for(i=0;i<n;i++)
    for(j=yzero;j<yR;j++)
      temp[i][j]=Tmt[i][j];
  for(i=xm;i>=0;i--)
    for(j=yzero;j<=yR;j++)
      {
	ii=n-1-i;
	if(Tmt[i][j]==10)//type not identified;
	  {
	    nb_3=0;
	    nb_4=0;
	    for(iii=i-1;iii<=i+1;iii++)
	      for(jjj=j-1;jjj<=j+1;jjj++)
		{
		  if(Tmt[iii][jjj]==3)//(Cell[i][j]==Cell[iii][jj] && Tmt[iii][jjj]==3)
		    nb_3++;
		  else if(Tmt[iii][jjj]==4)//(Cell[i][j]==Cell[iii][jj] && Tmt[iii][jjj]==4)
		    nb_4++;
		}
	    if(nb_3>=th)
	      {
		temp[i][j]=3;
		temp[ii][j]=3;
	      }
	    else if(nb_4>=th)
	      {
		temp[i][j]=4;
		temp[ii][j]=4;
	      }
	  }
      }
   for(i=0;i<n;i++)
    for(j=yzero;j<yR;j++)
      Tmt[i][j]=temp[i][j];
  //remaining membrane types
  for(i=xm;i>=0;i--)
    for(j=yzero;j<=yR;j++)
      {
	ii=n-1-i;
	if(Tmt[i][j]==10)//there is an unidentified membrane
	  {
	    if(Tc[i][j-1]==0 || Tc[i+1][j-1]==0)
	      {
		Tmt[i][j]=2;
		Tmt[ii][j]=2;
	      }

	    if(Tc[i][j+1]==0 || Tc[i-1][j+1]==0)
	      {
		Tmt[i][j]=1;
		Tmt[ii][j]=1;
	      }
	  }
      }

  

  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)//cell wall
	  Tct[i][j]=20;//to get different colour there
      }
 
  
  //check symmetry initial tissue layout
  for(j=0;j<m;j++)
    for(i=0;i<(halfn-1);i++)
      {
	ii=n-1-i;
	if(Tmt[i][j]!=Tmt[ii][j])//if(Tc[i][j]!=Tc[ii][j])
	  {
	    printf("something is wrong in tissue layout at %i %i and %i %i celltype is %i and %i\n",i,j,n-1-i,j,Tct[i][j],Tct[n-1-i][j]);
	  } 
      } 
  //printf("finished checking initial tissue layout\n");
  //exit(1); 
  
  
}


void NumberCells()
{
  int i,j;
  int q=0;

  //nummer de cellen
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      Cell[i][j]=-1;

  int count=0;
  int nbs;
  int ii,jj;
  int dist;
  for(j=0;j<m;j++)
    for(i=0;i<n;i++)
	{
	  if(Tc[i][j]==2 || Tc[i][j]==1)//cell membrane or cytosol
	    { 
	      if(j>=yLRC1)
		{
		  if((Cell[i-1][j]==-1 && Cell[i][j-1]==-1)&&(Cell[i+1][j]==-1 && Cell[i][j+1]==-1))
		    {
		      Cell[i][j]=q;
		      //printf("location is %i %i; cell nr is %i\n",i,j,q);
		      q++;
		    }
		  else
		    {
		      if(Cell[i-1][j]!=-1)
			Cell[i][j]=Cell[i-1][j];
		      else if(Cell[i][j-1]!=-1)
			Cell[i][j]=Cell[i][j-1];
		      else if(Cell[i+1][j]!=-1)
			Cell[i][j]=Cell[i+1][j];
		      else if(Cell[i][j+1]!=-1)
			Cell[i][j]=Cell[i][j+1];
		    }
		}
	      else
		{
		  nbs=0;
		  dist=1;
		  for(ii=i-dist;ii<=i+dist;ii++)
		    for(jj=j-dist;jj<=j+dist;jj++)
		      {
			if(!((ii==i-1 && jj==j-1) || (ii==i+1 && jj==j-1)))
			  {
			    if(Cell[ii][jj]!=-1)
			      {
				Cell[i][j]=Cell[ii][jj];
				nbs++;
				count++;
				ii=i+1000;
				jj=j+1000;
			      }
			  }
		      }
		  jj=j-1;
		  ii=i;
		  while(ii<i+10 && Tc[ii][j]!=0)
		    {
		      if(Cell[ii][jj]!=-1)
			{
			  Cell[i][j]=Cell[ii][jj];
			  count++;
			  nbs++;
			  break;
			}
		      else
			ii++;
		    }

		  if(nbs==0)
		    {
		      Cell[i][j]=q;
		      //printf("location is %i %i; cell nr is %i\n",i,j,q);
		      q++;
		      count++;
		    }   
		}
	    }
	}

  actcells=q;

  //printf("nrcells is %i\n",actcells);
}

void FindCellCorners()
{
  int i,j,q;

  //find mini,maxi,minj,maxj,width,height of cells
  for(q=0;q<totcells;q++)
    {
      CellMinI[q]=10000;
      CellMinJ[q]=10000;
      CellMaxI[q]=0;
      CellMaxJ[q]=0;
      CellWidth[q]=0;
      CellHeight[q]=0;
    }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	q=Cell[i][j];
	if(q!=-1)
	  {
	    if(i<CellMinI[q])
	      CellMinI[q]=i;
	    else if(i>CellMaxI[q])
	      CellMaxI[q]=i;
	    if(j<CellMinJ[q])
	      CellMinJ[q]=j;
	    else if(j>CellMaxJ[q])
	      CellMaxJ[q]=j;
	  }
      }

  //non straigth cells? major and minor axis length instead?, centroid?
  for(q=0;q<actcells;q++)
    {
      CellWidth[q]=CellMaxI[q]-CellMinI[q]+1;
      CellHeight[q]=CellMaxJ[q]-CellMinJ[q]+1;

      //printf("cell %i mini %i maxi %i width %i minj %i maxj %i height %i\n",q,CellMinI[q],CellMaxI[q],CellWidth[q],CellMinJ[q],CellMaxJ[q],CellHeight[q]);
    }
  //exit(1);

  //printf("cell corner finding works\n");
}


void FindCellCentroid()
{
}

void FindCellSizes()
{
  int i,j,q;
  
  //clean arrays
  for(q=0;q<totcells;q++)
    {
      CellSizes[q]=0;
      InvCellSizes[q]=0;
    }
  
  //sla de grootte op van alle cellen  
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Cell[i][j]>=0)
	  CellSizes[Cell[i][j]]++;
	}
  
  //for(q=0;q<actcells;q++)
    //printf("cell nr is %i mini %i maxi %i minj %i maxj%i size is %i\n",q,CellMinI[q],CellMaxI[q],CellMinJ[q],CellMaxJ[q],CellSizes[q]);

  for(q=0;q<actcells;q++)
    InvCellSizes[q]=1.0/(double)CellSizes[q];

  //printf("cell size finding works\n");
}

void SetUpPrePattern()
{
  int i,j;
  double K=0.35*m;

  double bc1,bc2,bc3,bc4,bc5,bc6;//first intersection with "x-axis"
  double ec1,ec2,ec3,ec4,ec5,ec6;//second intersection with "x-axis"
  double ac1,ac2,ac3,ac4,ac5,ac6,ac7;
  int xm;
  int dc1,dc2,dc3,dc4,dc5,dc6;

  
  bc1=0;
  ec1=(n-1);
  bc2=(bc1+sizecap+1);
  ec2=(ec1-(sizecap+1));
  bc3=(bc2+sizecap+1);
  ec3=(ec2-(sizecap+1));
  bc4=(bc3+sizeepi+1);
  ec4=(ec3-(sizeepi+1));
  bc5=(bc4+sizecort+1);
  ec5=(ec4-(sizecort+1));
  bc6=(bc5+sizeendo+1);
  ec6=(ec5-(sizeendo+1));
  xm=(halfn-1);
  dc1=(300/xstep);
  dc2=(207/xstep);
  dc3=(142/xstep);
  dc4=(100/xstep);
  dc5=(88/xstep);
  dc6=(76/xstep);
  ac1=dc1/(pow((bc1-xm),2));
  ac2=dc2/(pow(abs(bc2-xm),3));
  ac3=dc3/(pow((bc3-xm),4));
  ac4=dc4/(pow(abs(bc4-xm),5));
  ac5=dc5/(pow((bc5-xm),6));
  ac6=dc6/(pow(abs(bc6-xm),7));
  ac7=10*dc6/(pow(abs(bc6-xm),7));


  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	PinPrePat[i][j]=0;
	Lax3PrePat[i][j]=0;
	Lax2PrePat[i][j]=0;
	Aux1PrePat[i][j]=0;
      }


#ifdef POLAR
  int ii;
  for(i=xm;i>=0;i--)
    for(j=0;j<m;j++)
      {
	ii=n-1-i;
	if(Tc[i][j]==2)//membrane
	  {
	    if(Tct[i][j]<=4)//lateral root cap
	      {

		if(Tmt[i][j]==1)// && Tc[i][j+2]!=3)//up
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		else if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
	      }
	    else if(Tct[i][j]==5)//epidermis
	      {

		if(Tmt[i][j]==1)//up
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		else if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
	      }
	    else if(Tct[i][j]==6)//cortex
	      {
		
		if(Tmt[i][j]==1 && j>YR)//up
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		else if(Tmt[i][j]==2 && j<YR)//down
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		else if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
	      }
	    else if(Tct[i][j]==7)//endodermis
	      {
		if(Tmt[i][j]==2)//down
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		else if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.35;
		    PinPrePat[ii][j]=0.35;
		  }
	      }
	    else if(Tct[i][j]==8)//pericycle
	      {
		if(j>(int)(arad4*i+brad4))
		  {
		    if(Tmt[i][j]==2)//down
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		   else if(Tmt[i][j]==3)//in, EXTRA????!!!
		      {
			PinPrePat[i][j]=0.1;
			PinPrePat[ii][j]=0.1;
		      }
		  }
		else
		  {
		    if(Tmt[i][j]==4)//(down is defined as out here, due to circularity)
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		  }
	      }
	    else if(Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11)//vasculature
	      {
		if(Tmt[i][j]==2)//down
		  {
		   
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		else if(Tmt[i][j]==4)//out //EXTRA, TO ENHANCE AUXIN IN PERICYCLE AND PROTOXYLEM
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
	      }
	    else if(Tct[i][j]==12)//QC
	      {
		if(Tmt[i][j]==4)//(down is defined as out here, due to circularity)
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
	      }
	    else if(Tct[i][j]>12)//columella
	      {
		PinPrePat[i][j]=1;
		PinPrePat[ii][j]=1;
	      }
	   
	  }
      }
#endif

#ifdef APOLAR
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//membrane
	  {
	    PinPrePat[i][j]=1;
	  }
      }  
#endif


  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	Lax3PrePat[i][j]=0;
      }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	Lax2PrePat[i][j]=0;
      }
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
#ifdef AUX1INEPI
	if((Tct[i][j]!=6 && Tct[i][j]!=7) && (Tct[i][j]!=10 && Tct[i][j]!=15))//if(Tct[i][j]==5)
	  {
	    
	    Aux1PrePat[i][j]=1;
	    if(Tct[i][j]==5 && j<yMZEZ)
	      Aux1PrePat[i][j]=0.3;//0.8;
	  }
	else
#endif
	  Aux1PrePat[i][j]=0;
      }


  //check symmetry initial PIN layout
  /*
  for(j=0;j<m;j++)
    for(i=0;i<=(halfn)-1;i++)
      {
	if((int)(1000*PinPrePat[i][j])!=(int)(1000*PinPrePat[n-1-i][j]))
	  {
	    printf("something is wrong in PIN layout at %i %i and %i %i\n",i,j,n-1-i,j);
	  }
      }
  printf("finished checking initial PIN layout\n");
  exit(1);*/

  //check symmetry initial AUX1 layout
  /*for(j=0;j<m;j++)
    for(i=0;i<=(halfn)-1;i++)
      {
	if((int)(1000*Aux1PrePat[i][j])!=(int)(1000*Aux1PrePat[n-1-i][j]))
	  {
	    printf("something is wrong in AUX1 layout at %i %i and %i %i\n",i,j,n-1-i,j);
	  }
      }
  printf("finished checking initial AUX1 layout\n");
  exit(1);*/

  //check symmetry initial LAX2 layout
  /*for(j=0;j<m;j++)
    for(i=0;i<=(halfn)-1;i++)
      {
	if((int)(1000*Lax2PrePat[i][j])!=(int)(1000*Lax2PrePat[n-1-i][j]))
	  {
	    printf("something is wrong in LAX2 layout at %i %i and %i %i\n",i,j,n-1-i,j);
	  }
      }
  printf("finished checking initial LAX2 layout\n");
  exit(1);*/

  //check symmetry initial LAX3 layout
  /*
  for(j=0;j<m;j++)
    for(i=0;i<=(halfn)-1;i++)
      {
	if((int)(1000*Lax3PrePat[i][j])!=(int)(1000*Lax3PrePat[n-1-i][j]))
	  {
	    printf("something is wrong in LAX3 layout at %i %i and %i %i\n",i,j,n-1-i,j);
	  }
      }
  printf("finished checking initial LAX3 layout\n");
  exit(1);*/

}

void InitConditions()
{
  int i,j,k,q;

  //set gene expression levels:
  for(q=0;q<actcells;q++)
    {
      MeanCells[q]=100;
      ARF[q]=0;//
      GG[q]=0;
      AA[q]=0;
      BB[q]=0;
      CC[q]=0;
      DD[q]=0;
      Plt[q]=10;
      Lax3[q]=10;
      Plts[q]=0;//10;
      
      if(CellMaxJ[q]<YR)
	{
	  if(CellMaxJ[q]<YC)
	    Plts[q]=80;
	  else 
	    Plts[q]=50;
	}
      
#ifdef PINCONSTANT
      Pin[q]=100;//50;//100;
#endif
#ifdef PINREGULATED
      Pin[q]=30;//10;
#endif
      Yuc3[q]=50;//100;
      Gh33[q]=50;//100;
      Lax2[q]=50;//100;
      Aux1[q]=30;//100;

      TimePrevGrowth[q]=0;
      //PrimeDiff[q]=0;
      Diff[q]=0;
      CellZonationState[q]=0;
    }
  //set auxin concentration levels:
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]!=3)
	   {
	     if(j<10)
	       Caux1[i][j]=3500;//4500;
	     else if(j<30)
	       Caux1[i][j]=1700;//2000;
	     else if(j<60)
	      Caux1[i][j]=900;
	     else if(j<100)
	      Caux1[i][j]=400;
	     else if(j<250)
	      Caux1[i][j]=60;
	     else if(j<500)
	       Caux1[i][j]=20;
	     else
	       Caux1[i][j]=0.5;//0.1
	  }
	//Caux1[i][j]=0.1;
      }

  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	DivRate[i][j]=0;
	GrowthRate[i][j]=0;
      }

}

