#include "Header.hh"


void dsolvetridiag(int ni)
{
  /**
   * ni - number of equations
   * a - sub-diagonal (means it is the diagonal below the main diagonal) -- indexed from 1..n-1
   * b - the main diagonal
   * c - sup-diagonal (means it is the diagonal above the main diagonal) -- indexed from 0..n
   * rhs - right part
   * x - the answer
   */
  int i;
  double mm;
  
  for (i = 1; i < ni; i++)
    {
      mm = a[i]/b[i-1];
      b[i] = b[i] - mm*c[i-1];
      rhs[i] = rhs[i] - mm*rhs[i-1];
    }

  res[ni-1] = rhs[ni-1]/b[ni-1];
  
  for (i = ni - 2; i >= 0; i--)
    res[i]=(rhs[i]-c[i]*res[i+1])/b[i];
  
  for(i=0;i<ni;i++)
    rhs[i]=res[i];
}


void UpdateAuxin()
{

  int i,j;
  static int Direction=0;
  int q;
  double func;

  if(Direction==0)//Implicit integration in j direction
    {             //Explicit integration in i direction
      Direction=1;
      
      //do one direction implicit one explicit
      for(i=0;i<n;i++)
	{
	  for(j=0;j<m;j++)
	    {
	      //Compute coefficients for implicit diffusion and fluxes
	      //diffusion only

	      if(Tc[i][j]==3)
		{
		  a[j]=0;
		  c[j]=0;
		  b[j]=1;
		  rhs[j]=0;
		}
	      else   
		{
		  a[j]=-DiffAuxC[i][j][3];//j-1
		  c[j]=-DiffAuxC[i][j][2];//j+1 
		  b[j]=1-(a[j]+c[j]);//j 
		  if(Tc[i][j]==0)
		    {
		      //influx
		      b[j]+=-0.5*tstep*(Inf[i][j][2]+Inf[i][j][3]);
		      //efflux
		      a[j]+=-0.5*tstep*Eff[i][j][3];
		      c[j]+=-0.5*tstep*Eff[i][j][2];
		    }
		  else if(Tc[i][j]==2)
		    {
		      //influx
		      a[j]+=-0.5*tstep*Inf[i][j][3];
		      c[j]+=-0.5*tstep*Inf[i][j][2];
		      //efflux
		      b[j]+=-0.5*tstep*(Eff[i][j][2]+Eff[i][j][3]);
		    }
		  
		  //explicit diffusion and fluxes
		  rhs[j]=Caux1[i][j]+DiffAuxC[i][j][0]*(Caux1[i+1][j]-Caux1[i][j])
		    +DiffAuxC[i][j][1]*(Caux1[i-1][j]-Caux1[i][j]);
		  if(Tc[i][j]==0)
		    {
		      rhs[j]+=0.5*tstep*(Eff[i][j][0]*Caux1[i+1][j]+Eff[i][j][1]*Caux1[i-1][j]);
		      rhs[j]+=0.5*tstep*((Inf[i][j][0]+Inf[i][j][1])*Caux1[i][j]);
		    }
		  else if(Tc[i][j]==2)
		    {
		      rhs[j]+=0.5*tstep*((Eff[i][j][0]+Eff[i][j][1])*Caux1[i][j]);
		      rhs[j]+=0.5*tstep*(Inf[i][j][0]*Caux1[i+1][j]+Inf[i][j][1]*Caux1[i-1][j]); 
		    }
		}//else: not Tc[i][j]==3
	    }
	  //Compute implicit diffusion and use this to update rhs
	  //Use new rhs to update volt2
	  dsolvetridiag(m); 
	  for(j=0;j<m;j++)
	    {
	      if(Tc[i][j]!=3)
		{
		  func=0.5*tstep*(paux[i][j]-daux[i][j]*Caux1[i][j]);
		  Cauxint[i][j]=rhs[j]+func;
		}
	    }
	}
      
    }
  if(Direction==1)//Implicit integration in i direction
    {             //Explicit integration in j direction
      Direction=0;
      
      //do one direction implicit one explicit
      for(j=0;j<m;j++)
	{
	  for(i=0;i<n;i++)
	    {
	      //Compute coefficients for implicit diffusion and fluxes
	      //diffusion only
	      if(Tc[i][j]==3)
		{
		  a[i]=0;
		  c[i]=0;
		  b[i]=1;
		  rhs[i]=0;
		}
	      else
		{
		  a[i]=-DiffAuxC[i][j][1];//i-1 direction
		  c[i]=-DiffAuxC[i][j][0];//i+1 direction
		  b[i]=1-(a[i]+c[i]);//i;
		  if(Tc[i][j]==0)
		    {
		      //influx
		      b[i]+=-0.5*tstep*(Inf[i][j][0]+Inf[i][j][1]);
		      //efflux
		      a[i]+=-0.5*tstep*Eff[i][j][1];
		      c[i]+=-0.5*tstep*Eff[i][j][0];
		    }
		  else if(Tc[i][j]==2)
		    {
		      //influx
		      a[i]+=-0.5*tstep*Inf[i][j][1];
		      c[i]+=-0.5*tstep*Inf[i][j][0];
		      //efflux
		      b[i]+=-0.5*tstep*(Eff[i][j][0]+Eff[i][j][1]);
		    }
		  
		  
		  //explicit diffusion and fluxes
		  rhs[i]=Cauxint[i][j]+DiffAuxC[i][j][2]*(Cauxint[i][j+1]-Cauxint[i][j])
		    +DiffAuxC[i][j][3]*(Cauxint[i][j-1]-Cauxint[i][j]);
		  if(Tc[i][j]==0)
		    {
		      rhs[i]+=0.5*tstep*(Eff[i][j][2]*Cauxint[i][j+1]+Eff[i][j][3]*Cauxint[i][j-1]);
		      rhs[i]+=0.5*tstep*((Inf[i][j][2]+Inf[i][j][3])*Cauxint[i][j]);
		    }
		  else if(Tc[i][j]==2)
		    {
		      rhs[i]+=0.5*tstep*((Eff[i][j][2]+Eff[i][j][3])*Cauxint[i][j]);
		      rhs[i]+=0.5*tstep*(Inf[i][j][2]*Cauxint[i][j+1]+Inf[i][j][3]*Cauxint[i][j-1]);
		    }
		}
	    }
	  //Compute implicit diffusion and use this to update rhs
	  //Use new rhs to update volt2
	  dsolvetridiag(n); 
	  for(i=0;i<n;i++)
	    { 
	      if(Tc[i][j]!=3)
		{
		  func=0.5*tstep*(paux[i][j]-daux[i][j]*Cauxint[i][j]);
		  Caux2[i][j]=rhs[i]+func;
		}
	    }
	}
    }

  //bereken de gemiddelde auxine conc per cell
  for(q=0;q<totcells;q++)
    MeanCells[q]=0;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Cell[i][j]!=-1)
	  {
	    q=Cell[i][j];
	    MeanCells[q]+=Caux1[i][j];
	  }
      }
  for(q=0;q<totcells;q++)
    MeanCells[q]=MeanCells[q]*InvCellSizes[q];
  
  //final update for next timestep
  //doe dit pas na het berekenen van het 
  //gemiddelde auxin level per cell
  //hierdoor doe je genexpressie update
  //nog op basis van oude aux1, net als
  //fluxes, diffusie, decay etc.    
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]!=3)
	  Caux1[i][j]=Caux2[i][j];
      }



  
  //check symmetry auxin levels
#ifndef ASYMMETRIC
  static int counter=0;
  static int counter_2 = 0;
  //printf("time %i value %f %f %f %f\n",counter,Caux1[42][1527],Caux1[42][1526],Caux1[43][1527],Caux1[43][1526]);
  for(j=0;j<m;j++)
    for(i=0;i<=(n/2)-1;i++)
      {
	if(Tc[i][j]!=3 && counter_2 == 0)
	  {
	    if((int)(1000*(Caux1[i][j]-Caux1[n-1-i][j]))!=0)
	      {
		printf("something is wrong in auxin levels at %i %i %i (%.10f %i %i) and  %i %i %i(%.10f %i %i)\n",i,j,Cell[i][j],Caux1[i][j],(int)(10000*Caux1[i][j]),Tc[i][j],n-1-i,j,Cell[n-1-i][j],Caux1[n-1-i][j],(int)(10000*Caux1[n-1-i][j]),Tc[n-1-i][j]);
		//printf("n is %i m is %i\n",n,m);
		//printf("halfway is %i\n",xm);
		//printf("yzero %i and yend %i\n",yzero,yEND);
		//printf("nearest celltype is %i\n",Tct[i-1][j-1]);
		//printf("totalcellnr is %i\n",actcells);
		printf("counter is %i\n",counter);
		counter_2=1;
		//exit(1);
	      }
	  }
      }
  //printf("finished checking symmetry auxin levels, counter is %i\n",counter);
  counter++;
  //exit(1);
#endif
  
}



void UpdateGeneExpression(int t)
{
  int q,k;
  static double sat2plt=satplt*satplt;
  static double sat2gg=satgg*satgg;
  static double sat2pin=satpin*satpin;
  static double sat2plts=satplts*satplts;
  static double sat2yuc3=satyuc3*satyuc3;
  static double sat2gh33=satgh33*satgh33;
  static double sat2aux1=sataux1*sataux1;

  double dtARF,dtAA,dtBB,dtCC,dtDD,dtPlt,dtPlts;
  double dtPin,dtAux1,dtLax3;             
  double dtYuc3,dtGh33,dtPP;
  static double tspeedupstep=tstep*speedup2;
 
  //update gene expression levels: cell based
  for(q=0;q<actcells;q++)//for(q=0;q<totcells;q++)
    {
      dtARF=0;
      dtAA=0;
      dtBB=0;
      dtCC=0;
      dtDD=0;
      dtPlt=0;
      dtPlts=0;
      dtPP=0;
      dtPin=0;
      dtAux1=0;
      dtYuc3=0;
      dtGh33=0;
      dtLax3=0;


      //ARF depends on auxin
      double ass=0.01*0.3;
      double diss=0.001;//0.25;
      double pI=0.01;
      double dIbasal=0.0001;
      double dITIR1=0.001;
      double ARFtotal=100;
      dtARF=tspeedupstep*(diss*(ARFtotal-ARF[q])-ass*ARF[q]*(pI/(dIbasal+dITIR1*MeanCells[q])));
           
      double satA=40;//50;
      double satB=40;//50;
      double satC=40;//50;
      double satD=40;//58;
      dtAA=tspeedupstep*(maxgg*ARF[q]*ARF[q]/(ARF[q]*ARF[q]+satA*satA )-decaygg*AA[q]);
      dtBB=tspeedupstep*(maxgg*AA[q]*AA[q]/(AA[q]*AA[q]+satB*satB )-decaygg*BB[q]);
      dtCC=tspeedupstep*(maxgg*BB[q]*BB[q]/(BB[q]*BB[q]+satC*satC )-decaygg*CC[q]);
      dtDD=tspeedupstep*(maxgg*CC[q]*CC[q]/(CC[q]*CC[q]+satD*satD )-decaygg*DD[q]);

     
      //plt depends on??
      dtPlt=0;
      
      if(CellMinJ[q]>yLRCstraight+15*HMZ+15)
	{
	  if(Tct[CellMinI[q]][CellMinJ[q]]==8)
	    {
	      double maxpp=0.01;
	      double decaypp=0.0001;
	      double satppaux=80;
	      double satpppp=80;
	      double temp1;
	      double temp2;
	      double auxinfac;
	      double ppfac;
	      double latinh;
	      int qqup;
	      int qqdown;
	      temp1=MeanCells[q]*MeanCells[q]*MeanCells[q]*MeanCells[q];
	      temp2=satppaux*satppaux*satppaux*satppaux;
	      auxinfac=temp1/(temp1+temp2);
	      temp1=PP[q]*PP[q]*PP[q]*PP[q];
	      temp2=satpppp*satpppp*satpppp*satpppp;
	      ppfac=temp1/(temp1+temp2);
	      //if(PP[q]>40)
	      //ppfac=1;
	      
	      qqup=Cell[CellMinI[q]][CellMaxJ[q]+2];
	      qqdown=Cell[CellMinI[q]][CellMinJ[q]-2];
	      latinh=1;//PP[q]/(0.5*(PP[qqup]+PP[qqdown]));
	      if(latinh>1.5)
		latinh=1.5;
	      if(latinh<0.5)
		latinh=0.5;
	      dtPP=tspeedupstep*(maxpp*(auxinfac+ppfac)-decaypp*PP[q]);
	      //dtPP=tspeedupstep*(maxpp*(max(auxinfac,ppfac)*latinh)-decaypp*PP[q]);
	    }
	}
#ifndef RESTART
      if(t<70000)
	{
	  dtPlts=tspeedupstep*(maxplts*ARF[q]*ARF[q]/(ARF[q]*ARF[q]+sat2plts)-decayplts*Plts[q]);
	  Pltsexpr[q]=100*ARF[q]*ARF[q]/(ARF[q]*ARF[q]+sat2plts);
	}
      else
#endif
	{
	  double temp;
	  temp=(DD[q]*DD[q]/(DD[q]*DD[q]+sat2plts));
	  dtPlts=tspeedupstep*(maxplts*temp-decayplts*Plts[q]);
	  Pltsexpr[q]=100*(temp);		   
	}
     
     
#ifdef PINREGULATED 
      if(Tct[(int)(0.5*(CellMinI[q]+CellMaxI[q]))][(int)(0.5*(CellMinJ[q]+CellMaxJ[q]))]<=7) 
		dtPin=tspeedupstep*(0.1*maxpin+0.9*maxpin*(MeanCells[q]*MeanCells[q]/(MeanCells[q]*MeanCells[q]+km_pin2*km_pin2))-decaypin*Pin[q]);
		//dtPin=tspeedupstep*(0.1*maxpin+0.9*maxpin*(ARF[q]*ARF[q]/(ARF[q]*ARF[q]+3*3))-decaypin*Pin[q]);
	  else if(Tct[(int)(0.5*(CellMinI[q]+CellMaxI[q]))][(int)(0.5*(CellMinJ[q]+CellMaxJ[q]))]>7 && Tct[(int)(0.5*(CellMinI[q]+CellMaxI[q]))][(int)(0.5*(CellMinJ[q]+CellMaxJ[q]))]<=11)
		//dtPin=tspeedupstep*(0.1*maxpin+0.9*maxpin*(ARF[q]*ARF[q]/(ARF[q]*ARF[q]+2*2))-decaypin*Pin[q]);
		dtPin=tspeedupstep*(0.3*maxpin+0.7*maxpin*(MeanCells[q]*MeanCells[q]/(MeanCells[q]*MeanCells[q]+km_pin1*km_pin1))-decaypin*Pin[q]);
      else
		dtPin=0;
#endif
      /*
      if(Tct[(int)(0.5*(CellMinI[q]+CellMaxI[q]))][(int)(0.5*(CellMinJ[q]+CellMaxJ[q]))]>7 && Tct[(int)(0.5*(CellMinI[q]+CellMaxI[q]))][(int)(0.5*(CellMinJ[q]+CellMaxJ[q]))]<=11)
	dtPin=tspeedupstep*(0.3*maxpin+0.7*maxpin*(85*85/(MeanCells[q]*MeanCells[q]+85*85))-decaypin*Pin[q]);
      else
	dtPin=0;
      */
      //Pin,Yuc3,Gh33,Lax2 all depend on pltslow
#ifdef YUC3GH33REGULATED
      dtYuc3=tspeedupstep*(maxyuc3*Plts[q]*Plts[q]/(Plts[q]*Plts[q]+sat2yuc3)-decayyuc3*Yuc3[q]);
      dtGh33=tspeedupstep*(maxgh33*sat2gh33/(Plts[q]*Plts[q]+sat2gh33)-decaygh33*Gh33[q]);
#endif


#ifdef AUX1REGULATED
      double aa=0;//0.5;//0.7;//0.;
      double bb=1-aa;
      int kk=(48/xstep);
      double sizedep=(aa*(kk*kk/(kk*kk+CellHeight[q]*CellHeight[q]))+bb);
      double ckfac;
      int meani=CellMinI[q]+2;//(int)(0.5*(CellMinI[q]+CellMaxI[q]));
      int meanj=CellMinJ[q]+2;//(int)(0.5*(CellMinJ[q]+CellMaxJ[q]));
      if(Tct[meani][meanj]>=1 && Tct[meani][meanj]<=4)//lrc
	ckfac=1.5;
      else if(Tct[meani][meanj]>=8 && Tct[meani][meanj]<=11)//peri & vasc
	{
	  if(CellZonationState[q]>50)//if(CellZonationState[q]>30)
	    ckfac=1.5;
	  else
	    ckfac=0.75;
	}//should have been an else here!
      ckfac=0.75;
      ckfac=1;
      double auxindep=(MeanCells[q]*MeanCells[q]/(MeanCells[q]*MeanCells[q]+(ckfac*km_aux1)*(ckfac*km_aux1)));

      /*
      if(CellMinJ[q]>yLRC1-4*HMZ-4)
	{
	  if(CellMinI[q]<xm)
	    {
	      if(Tct[CellMinI[q]][CellMinJ[q]]>=8 && Tct[CellMinI[q]][CellMinJ[q]]<=11)//pericycle
		{
		  auxindep=2.0*(MeanCells[q]*MeanCells[q]/(MeanCells[q]*MeanCells[q]+125*125));
		}
	    }
	  else
	    {
	      if(Tct[CellMaxI[q]][CellMinJ[q]]==8  && Tct[CellMaxI[q]][CellMinJ[q]]<=11)//pericycle
		{
		  auxindep=2.0*(MeanCells[q]*MeanCells[q]/(MeanCells[q]*MeanCells[q]+125*125));
		}
	    }
	}
      */
      //double auxindep=MeanCells[q]*MeanCells[q]*MeanCells[q]*MeanCells[q]/(MeanCells[q]*MeanCells[q]*MeanCells[q]*MeanCells[q]+km_aux1*km_aux1*km_aux1*km_aux1);
      
     // if(CellMinJ[q]>yLRCstraight && Tct[(int)(0.5*(CellMinI[q]+CellMaxI[q]))][(int)(0.5*(CellMinJ[q]+CellMaxJ[q]))]>=8 && Tct[(int)(0.5*(CellMinI[q]+CellMaxI[q]))][(int)(0.5*(CellMinJ[q]+CellMaxJ[q]))]<=11)
	  if(CellMinJ[q]>yLRCstraight && Tct[CellMinI[q]+2][CellMinJ[q]+2]>=8 && Tct[CellMinI[q]+2][CellMinJ[q]+2]<=11)
	{
	  auxindep=(MeanCells[q]*MeanCells[q]/(MeanCells[q]*MeanCells[q]+75*75));
	  //double auxindep2=(MeanCells[q]*MeanCells[q]/(MeanCells[q]*MeanCells[q]+90*90));
	
	  //dtAux1=tspeedupstep*(maxaux1*auxindep+maxaux1*auxindep2-decayaux1*Aux1[q]);
	}
      //else
      dtAux1=tspeedupstep*(maxaux1*sizedep*auxindep-decayaux1*Aux1[q]);
#endif




    
   
   ARF[q]+=dtARF;
   AA[q]+=dtAA;
   BB[q]+=dtBB;
   CC[q]+=dtCC;
   DD[q]+=dtDD;
   Plt[q]+=dtPlt;
   Plts[q]+=dtPlts;
   PP[q]+=dtPP;
   Pin[q]+=dtPin;
   Aux1[q]+=dtAux1;
   Lax3[q]+=dtLax3;
   Yuc3[q]+=dtYuc3;
   Gh33[q]+=dtGh33;

}

#ifdef PLASMODESMATAOPEN
  int i,j,ii,jj;
  double pdflux=4*0.01/(double)xstep;
  //initialise PLTs on grid
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Cell[i][j]!=-1)//cell there
	  {
	    q=Cell[i][j];
	    Pltsgrid[i][j]=Plts[q];
	    Pltsgrid2[i][j]=Plts[q];
	  }
	else
	  {
	    Pltsgrid[i][j]=0;
	    Pltsgrid2[i][j]=0;
	  }
      }
 
  //PLTs fluxes between cells through plasmodesmata
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//mebrane, so point of possible exchange
	  {
	    if(Tc[i-1][j]==0)//wall to left, so left membrane
	      {
		ii=i-1;
		while(ii>0 && (i-ii)<4)
		  {
		    ii--;
		    if(Tc[ii][j]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[ii][j]-Pltsgrid[i][j]);
			break;
		      }
		  }
	      }
	    if(Tc[i+1][j]==0)//wall to right, so right membrane
	      {
		ii=i+1;
		while(ii<n && (ii-i)<4)
		  {
		    ii++;
		    if(Tc[ii][j]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[ii][j]-Pltsgrid[i][j]);
			break;
		      }
		  }
	      }
	    if(Tc[i][j-1]==0)//wall above, so apical membrane ???????
	      {
		jj=j-1;
		while(jj>0 && (j-jj)<4)
		  {
		    jj--;
		    if(Tc[i][jj]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[i][jj]-Pltsgrid[i][j]);
			break;
		      }		 
		  }
	      }
	    if(Tc[i][j+1]==0)//wall below, so basal membrane ?????????
	      {
		jj=j+1;
		while(jj<m && (jj-j)<4)
		  {
		    jj++;
		    if(Tc[i][jj]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[i][jj]-Pltsgrid[i][j]);
			break;
		      }		 
		  }
	      }
	  }
      }
#ifdef PLTLEAKAGETOMAINROOT
  double Pltslevelshoot=0;
  double pdfluxshoot=pdflux;
  for(i=0;i<n;i++)
    for(j=m-1;j>=0;j--)
      {
	if(Cell[i][j]!=-1)//topmost part of topmost cell found
	  {
	    Pltsgrid2[i][j]+=pdfluxshoot*(Pltslevelshoot-Pltsgrid[i][j]);
	    j=-100;//so break from second for loop
	  }
      }
#endif
  
    
  //transform back from grid to cellular
  //assumption that within cell PLTs differences
  //even out quickly through diffusion
  for(q=0;q<totcells;q++)
    Plts[q]=0;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Cell[i][j]!=-1)//cell there
	  {
	    q=Cell[i][j];
	    Plts[q]+=Pltsgrid2[i][j];
	  }
      }
  for(q=0;q<actcells;q++)
    Plts[q]*=(double)InvCellSizes[q];
 
#endif
}



void UpdateMembraneProteinsAndFluxes(int time)
{
  int i,j,k,q,ii;

  static int counter=0;
  FILE *f;
  double invspstep=1.0/(double)(xstep);

#ifndef ASYMMETRIC
  for(i=xm;i>=0;i--)
    for(j=0;j<m;j++)
#else
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
#endif   
      {
	ii=n-1-i;
	PinPrePat[i][j]=0;
#ifndef ASYMMETRIC
	PinPrePat[ii][j]=0;
#endif
	Aux1PrePat[i][j]=0;
#ifndef ASYMMETRIC
	Aux1PrePat[ii][j]=0;
#endif
      }


  //WHY NOT THROW AWAY THE WHOLE SETUPPREPATTERNS
  //YOU DO SIMILAR THINGS THERE AS HERE, SO IS KIND OF DOUBLE?
  //NOW IF YOU WANT TO CHANGE SOMETHING YOU NEED TO CHANGE IT IN TWO LOCATIONS, RISKY!

  //SAME HOLDS PERHAPS FOR POLAR AND APOLARMERISTEM?
  //PERHAPS MUCH EASIER TO DEFINE WHOLE THING FOR POLAR MERISTEM
  //AND THEN ADD SOME EXTRA LATERAL PINS IF APOLARMERISTEM IS DEFINED?

  //HERE, FIRST TIME YOU NEED TO SET UP THE ZONATION, NOW IT 
  //DOES NOT WORK PROPERLY FIRST TIME BECAUSE YOU CALL IT
  //FROM INITPLANT PRIOR TO HAVING CALLED GROWTH FUNCTION

  if(time==0)
    {
      for(q=0;q<actcells;q++)
	{
	  if(CellMaxJ[q]<yR)
	    CellZonationState[q]=0;
	  else if(CellMaxJ[q]<yLRC1)
	    CellZonationState[q]=30;
	  else if(CellMaxJ[q]<yLRC1+7*HEZ+7)
	    CellZonationState[q]=60;
	  else
	    CellZonationState[q]=100;
	}
    }

#ifdef POLAR
  int kk,ll;
  int mindist;
  double disttresh;
  double distance; 
  
#ifndef ASYMMETRIC
  for(i=xm;i>=0;i--)
    for(j=0;j<m;j++)
    {
	ii=n-1-i;
#else
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
    {
		ii=i;
#endif  
      
	q=Cell[i][j];

	if(Tc[i][j]==2)//membrane
	  {
	    if(Tct[i][j]<=4) //lateral root cap
	      {		
		if(Tct[i][j]==1 && j>yLRCstraight) //lateral root cap
		  {
		    if(Tmt[i][j]==1)//up
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else if(Tmt[i][j]==3)//inward membrane
		      {
			disttresh=(int)(0.2*(CellMaxJ[Cell[i][j]]-CellMinJ[Cell[i][j]]));
			distance =10000;
			for(kk=max(0,i-100/xstep);kk<=min(n-1,i+100/xstep);kk++) //min max to not go outside the field
			  for(ll=CellMinJ[Cell[i][j]];ll<=CellMaxJ[Cell[i][j]];ll++)//cells always inside the field
			    {
			      if(Tmt[kk][ll]==1  && Cell[kk][ll]==Cell[i][j])//upward membrane of same cell
				{
				  distance=abs(kk-i)+abs(ll-j);
				  if(distance<=disttresh)
				    {
				      kk=10000;
				      ll=10000;//to stop searching if distance below threshold is found
				    }
				}
			    }
			if(distance<=disttresh)
			  {
			    PinPrePat[i][j]=1;
			    PinPrePat[ii][j]=1;
			  }
			else
			  {
			    PinPrePat[i][j]=0.1;
			    PinPrePat[ii][j]=0.1;
			  }
		      }
		  }
		else //other lateral root cap cells
		  {
#ifdef APOLARMERISTEM
		    if(Tmt[i][j]==1)//up
		      {
			if((i<xm && j>=(int)(arad1*i+brad1)) || (i>xm && j>=(int) (yR-arad1*(i-xm))))
			{
			PinPrePat[i][j]=3;//1;//WHY NOT 1???
			PinPrePat[ii][j]=3;//1;//1.5;
		    }
		    else
		    {
			PinPrePat[i][j]=1.5;//1;//WHY NOT 1???
			PinPrePat[ii][j]=1.5;//1;//1.5;
		    }
		      }
		    else if(Tmt[i][j]==3 && ((i<xm && j>=(int)(arad1*i+brad1)) || (i>xm && j>=(int) (yR-arad1*(i-xm)))))
		    {
				PinPrePat[i][j]=0.3;//0.25;//0.05;
				PinPrePat[ii][j]=0.3;//0.25;//0.05;
				
			}
		    else if(Tmt[i][j]==3 || Tmt[i][j]==4)//in and out
		      {
			PinPrePat[i][j]=0.05;//0.25;//0.05;
			PinPrePat[ii][j]=0.05;//0.25;//0.05;
		      } 
#else
		    if(Tmt[i][j]==1)//up
		      {
			PinPrePat[i][j]=0.5;
			PinPrePat[ii][j]=0.5;
		      }
#endif
		  } 
	      }
	    else if(Tct[i][j]==5)//epidermis different
	      {
		
		if(Tmt[i][j]==1)//up
		  {
		    if(j<yR)
		      {
			PinPrePat[i][j]=1;//
			PinPrePat[ii][j]=1;//
		      }
		    else if(j<yLRCstraight)
		      {
			
			PinPrePat[i][j]=0.5;
			PinPrePat[ii][j]=0.5;
		      }
		    else if(CellZonationState[q]<100)//untill EZ
		      {
			PinPrePat[i][j]=1;//
			PinPrePat[ii][j]=1;//
		      }
		    else//DZ
		      {
			PinPrePat[i][j]=0.35;
			PinPrePat[ii][j]=0.35;
		      }
		  }
#ifdef APOLARMERISTEM
		else if(Tmt[i][j]==2)//down
		  {
		    if(CellZonationState[q]==0)//PLEASE NOTE THAT I HAVE REDUCED SIZE OF THIS ZONE!
		      {
			if(j<yLRCstraight)//(i<xm && j<=(int)(arad1*i+brad1)) || (i>xm && j<=(int) (yR-arad1*i)))//(j<=(int)(arad1*i+brad1))
			  {
			    PinPrePat[i][j]=0.7;
			    PinPrePat[ii][j]=0.7;
			  } 
			else if(j<=yLRCstraight)
			  {
			    PinPrePat[i][j]=0.25;
			    PinPrePat[ii][j]=0.25;
			  } 
		      }
		  }
		else if(Tmt[i][j]==3 || Tmt[i][j]==4)//CODE FOR IN AND OUT IS VERY SIMILAR, CAN BE COMBINED?
		  {
		    if(CellZonationState[q]==0)
		      {
			if(j<yLRCstraight)//((i<xm && j<=(int)(arad1*i+brad1)) || (i>xm && j<=(int) (yR-arad1*i)))//(j<=(int)(arad1*i+brad1))
			  {
			    PinPrePat[i][j]=0.5;
			    PinPrePat[ii][j]=0.5;
			  } 
			else if(j<=yLRCstraight)
			  {
			    PinPrePat[i][j]=0.25;
			    PinPrePat[ii][j]=0.25;
			  }
			else
			  {
			    PinPrePat[i][j]=0.1;
			    PinPrePat[ii][j]=0.1;
			  }
		      } 
		    else
		      {
			if(Tmt[i][j]==3)
			  {
			    PinPrePat[i][j]=0.1;
			    PinPrePat[ii][j]=0.1;
			  }

			if(Tmt[i][j]==4 && CellMinJ[Cell[i][j]]<yR)
			  {
			    PinPrePat[i][j]=0.6;
			    PinPrePat[ii][j]=0.6;
			  }
			else if(Tmt[i][j]==4 && CellMinJ[Cell[i][j]]<yLRCstraight+10*HMZ+10)
			  {
			    PinPrePat[i][j]=0.3;
			    PinPrePat[ii][j]=0.3;
			  }
			else if(Tmt[i][j]==4 && CellMinJ[Cell[i][j]]<yLRC1)
			  {
			    PinPrePat[i][j]=0.1;
			    PinPrePat[ii][j]=0.1;
			  }
		      }
		  }
#else
		else if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }	    
#endif
	      }
	    else if(Tct[i][j]==6)//cortex different
	      {
		//normal
		if(Tmt[i][j]==1 && CellZonationState[Cell[i][j]]>50)//if(Tmt[i][j]==1 && CellZonationState[Cell[i][j]]>30)//up
		  {
		    if(CellZonationState[q]<100)//untill EZ
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else//DZ
		      {
			PinPrePat[i][j]=0.35;
			PinPrePat[ii][j]=0.35;
		      }
		    
		  }
		if(Tmt[i][j]==2 && CellZonationState[Cell[i][j]]<=50)//if(Tmt[i][j]==2 && CellZonationState[Cell[i][j]]<=30)//down
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }

		//always up
		/*
		if(Tmt[i][j]==1)
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		*/
		//always down
		/*
		if(Tmt[i][j]==2)
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		*/

#ifdef APOLARMERISTEM
		else if(Tmt[i][j]==3 || Tmt[i][j]==4)//CODE FOR IN AND OUT IS VERY SIMILAR, CAN BE COMBINED?
		  {
		    if(CellZonationState[q]==0)//PLEASE NOTE THAT I HAVE REDUCED SIZE OF THIS ZONE!
		      {
			if((j<yLRCstraight))//(i<xm && j<=(int)(arad1*i+brad1)) || (i>xm && j<=(int) (yR-arad1*i)))//(j<=(int)(arad1*i+brad1))
			  {
			    PinPrePat[i][j]=0.5;
			    PinPrePat[ii][j]=0.5;
			  } 
			else if(j<yLRCstraight)
			  {
			    PinPrePat[i][j]=0.25;
			    PinPrePat[ii][j]=0.25;
			  }
			else
			  {
			    PinPrePat[i][j]=0.1;
			    PinPrePat[ii][j]=0.1;
			  }
		      } 
		    else
		      {
			if(Tmt[i][j]==3)
			  {
			    PinPrePat[i][j]=0.1;
			    PinPrePat[ii][j]=0.1;
			  }
			/*
			if(CellZonationState[q]==100 && Tmt[i][j]==4)//in DZ
			  {
			    PinPrePat[i][j]=0.1;
			    PinPrePat[ii][j]=0.1;
			  }
			*/
		      }
		  }
#else
		else if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
#endif
		//different levels/patterns in DZ compared to EZ????
	      }
	    else if(Tct[i][j]==7)//endodermis different
	      {
		if(Tmt[i][j]==2)//down
		  {

		    //if(CellZonationState[q]<100)//untill EZ
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		      /*else
		      {
			PinPrePat[i][j]=0.5;
			PinPrePat[ii][j]=0.5;
			}*/
		  }
		/*else if(CellZonationState[q]==100 && Tmt[i][j]==1)//DZ
		  {
		    PinPrePat[i][j]=0.25;
		    PinPrePat[ii][j]=0.25;
		    }*/
#ifdef APOLARMERISTEM
		else if(Tmt[i][j]==1 && (j<yLRCstraight))//((i<xm && j<=(int)(arad1*i+brad1)) || (i>xm && j<=(int) (yR-arad1*i)))) //Up
		  {
		    PinPrePat[i][j]=0.5;//1;
		    PinPrePat[ii][j]=0.5;//1;
		  }
		else if(Tmt[i][j]==3 || Tmt[i][j]==4)//in and out
		  {
		    if(CellZonationState[q]==0)
		      {
			if(j<yLRCstraight)//((i<xm && j<=(int)(arad1*i+brad1)) || (i>xm && j<=(int) (yR-arad1*i)))//(j<=(int)(arad1*i+brad1))
			  {
			    PinPrePat[i][j]=0.7;
			    PinPrePat[ii][j]=0.7;
			  } 
			else
			  {  
			    if(Tmt[i][j]==3)//in
			      {
				PinPrePat[i][j]=0.5;
				PinPrePat[ii][j]=0.5;
			      }
			    else//out
			      {
				PinPrePat[i][j]=0.35;
				PinPrePat[ii][j]=0.35;
			      }
			  } 
		      }
		    else //PIN1 exp ends on MZ/EZ border
		      {
			if(Tmt[i][j]==3)//in
			  {
			    //if(CellZonationState[q]<100)//untill EZ
			      {
				PinPrePat[i][j]=0.35;
				PinPrePat[ii][j]=0.35;
			      }
			      /*else//DZ
			      {
				PinPrePat[i][j]=0.1;
				PinPrePat[ii][j]=0.1;
				}*/
			  }
			else//out
			  {
			    if((CellZonationState[q]==30 || CellZonationState[q]==50) && CellMinJ[q]<yLRC1-8*HMZ-8)//if(CellZonationState[q]==30 && CellMinJ[q]<yLRC1-8*HMZ-8)//BIT ARBITRARY?
			      {
				PinPrePat[i][j]=0.1;
				PinPrePat[ii][j]=0.1;
			      }
			    /*else if(CellZonationState[q]==100)
			      {
				PinPrePat[i][j]=0.1;
				PinPrePat[ii][j]=0.1;
				}*/
			    else
			      {
				PinPrePat[i][j]=0;
				PinPrePat[ii][j]=0;
			      }
			  }
		      }
		  }
#else
		else if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.35;
		    PinPrePat[ii][j]=0.35;
		  }
#endif
		//different levels/patterns in DZ compared to EZ????
	      }
	    else if(Tct[i][j]==8)//pericycle different
	      {
		if(Tmt[i][j]==2)//down
		  {

		    //if(CellZonationState[q]<100)//untill EZ
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		      /*else
		      {
			PinPrePat[i][j]=0.75;
			PinPrePat[ii][j]=0.75;
			}*/
		  }
#ifdef APOLARMERISTEM
		else if(Tmt[i][j]==3 || Tmt[i][j]==4)
		  {
		    if(CellZonationState[q]==0)
		      {
#ifndef ASYMMETRIC
			if(j<=(int)(arad1*i+brad1))
#else
			if(j<yLRCstraight)//((i<xm && j<=(int)(arad1*i+brad1)) || (i>xm && j>=(int) (yR-arad1*i)))
#endif
			  {
			    PinPrePat[i][j]=0.7;
			    PinPrePat[ii][j]=0.7;
			  }
			else
			  {
			    if(Tmt[i][j]==3)//in
			      {
				PinPrePat[i][j]=0.5;
				PinPrePat[ii][j]=0.5;
			      }
			    else//out
			      {
				PinPrePat[i][j]=0.35;
				PinPrePat[ii][j]=0.35;
			      }
			  }
		      }
		    else if(CellZonationState[q]<=50)
		      {
			/*
			if(Tmt[i][j]==3)//in
			  {
			    PinPrePat[i][j]=0.35;
			    PinPrePat[ii][j]=0.35;
			  }
			else if(Tmt[i][j]==4 && CellMinJ[q]<yLRC1-7*HMZ-7)//BIT ARBITRARY?
			  {
			    PinPrePat[i][j]=0.1;
			    PinPrePat[ii][j]=0.1;
			  }
			*/

			//to have it equal between MZ and EZ
			if(Tmt[i][j]==3)//in
			  {
			    PinPrePat[i][j]=0.1;
			    PinPrePat[ii][j]=0.1;
			  }
			
		      }
		    else
		      {
			//if(Tmt[i][j]==3)//in
			  {
			    PinPrePat[i][j]=0.1;
			    PinPrePat[ii][j]=0.1;
			  }
		      }
		  }
#else
		else if(Tmt[i][j]==3)//in, EXTRA????!!!
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
#endif
	      }	      
	    else if(Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11)//vasculature different
	      {
		if(Tmt[i][j]==2)//down REMOVED HERE THAT FOR APOLAR MERISTEM IT IS ONLY 0.75 IN LOWER PARTS
		  {
		    //if(CellZonationState[q]<100)//untill EZ
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		      /*else
		      {
			PinPrePat[i][j]=0.75;
			PinPrePat[ii][j]=0.75;
			}*/
		  }
#ifdef APOLARMERISTEM
	      else if(Tmt[i][j]==3 || Tmt[i][j]==4)
		{
		  if(CellZonationState[q]==0)
		    {
		      if(j<=(int)(arad1*i+brad1) && j<yLRCstraight)
			{
			  PinPrePat[i][j]=0.75;
			  PinPrePat[ii][j]=0.75;
			}
		      else if(j<yR)
			{
			  if(Tmt[i][j]==3)//in
			    {
			      PinPrePat[i][j]=0.4;//0.5;
			      PinPrePat[ii][j]=0.4;//0.5;
			    }
			  else//out
			    {
			      PinPrePat[i][j]=0.4;
			      PinPrePat[ii][j]=0.4;
			    }
			}
		      else//does not occur
			{
			  if(Tmt[i][j]==3)//in
			    {
			      PinPrePat[i][j]=0.2;//0.3;
			      PinPrePat[ii][j]=0.2;//0.3;
			    }
			  else//out
			    {
			      PinPrePat[i][j]=0.2;
			      PinPrePat[ii][j]=0.2;
			    }
			}
		    }
		  else
		    {
		      /*if(Tmt[i][j]==3 && CellZonationState[q]==30)
			{
			  PinPrePat[i][j]=0.2;
			  PinPrePat[ii][j]=0.2;
			}
			else
			{
			  PinPrePat[i][j]=0.1;
			  PinPrePat[ii][j]=0.1;
			}
		      */
		      if(Tmt[i][j]==4 || Tmt[i][j]==3)
			{
			  if(CellZonationState[q]<=50)
			    {
			      PinPrePat[i][j]=0.025;
			      PinPrePat[ii][j]=0.025;
			    }
			  else
			    {
			      PinPrePat[i][j]=0.0525;//0.15;
			      PinPrePat[ii][j]=0.0525;//0.15;
			    }
			}
		    }
		}
#else
	      else if(Tmt[i][j]==4)//out //EXTRA, TO ENHANCE AUXIN IN PERICYCLE AND PROTOXYLEM
		{
		  PinPrePat[i][j]=0.1;
		  PinPrePat[ii][j]=0.1;
		}
#endif
	      }
	    else if(Tct[i][j]==12)//QC, identical
	      {
		if(j==CellMinJ[Cell[i][j]])//works better than Tmt[i][j]==4
		  {
		    PinPrePat[i][j]=0.5;
		    PinPrePat[ii][j]=0.5;
		  }
	      }
	    else if(Tct[i][j]==13)//columella bottom layer,identical
	      {
		if(Tmt[i][j]<4)
		  {
		    
		    PinPrePat[i][j]=0.5;
		    PinPrePat[ii][j]=0.5;
		  }
	      }
	    else if(Tct[i][j]>13)//columella except bottom layer, identical
	      {
		if(Tct[i][j]<17)//not the topmost one
		  {
		    if(Tmt[i][j]<=3)//left,right and up
		      {
			PinPrePat[i][j]=0.75;
			PinPrePat[ii][j]=0.75;
		      }
		    else 
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		  }
		else
		  {
		    if(Tmt[i][j]==1)
		      {
			PinPrePat[i][j]=0.75;
			PinPrePat[ii][j]=0.75;
		      }
		    else
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		  }
	      }	   
	  }
      }
#endif

 
 
  /*
   for(i=xm;i>=0;i--)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//membrane
	  {
	    ii=n-1-i;
	    q=Cell[i][j];

	    if(Tct[i][j]>=1 && Tct[i][j]<=4)//lrc
	      {
		PinPrePat[i][j]=0;
		PinPrePat[ii][j]=0;

		if(Tct[i][j]==1 && j>yLRCstraight)//lrc 1 upper part
		  {
		    if(Tmt[i][j]==1)//up
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else if(Tmt[i][j]==3)//inward membrane
		      {
			disttresh=(int)(0.2*(CellMaxJ[Cell[i][j]]-CellMinJ[Cell[i][j]]));
			distance =10000;
			for(kk=max(0,i-100/xstep);kk<=min(n-1,i+100/xstep);kk++) //min max to not go outside the field
			  for(ll=CellMinJ[Cell[i][j]];ll<=CellMaxJ[Cell[i][j]];ll++)//cells always inside the field
			    {
			      if(Tmt[kk][ll]==1  && Cell[kk][ll]==Cell[i][j])//upward membrane of same cell
				{
				  distance=abs(kk-i)+abs(ll-j);
				  if(distance<=disttresh)
				    {
				      kk=10000;
				      ll=10000;//to stop searching if distance below threshold is found
				    }
				}
			    }
			if(distance<=disttresh)
			  {
			    PinPrePat[i][j]=1;
			    PinPrePat[ii][j]=1;
			  }
		      }
		  }
		else 
		  {
		    if(Tmt[i][j]==1)
		      {
			if(j<yR)
			  {
			    PinPrePat[i][j]=0.5;
			    PinPrePat[ii][j]=0.5;
			  }
			else
			  {
			    PinPrePat[i][j]=0.75;
			    PinPrePat[ii][j]=0.75;
			  }
		      }
		    else if(Tmt[i][j]==3 || Tmt[i][j]==4)
		      {
			
			PinPrePat[i][j]=0.1;
			PinPrePat[ii][j]=0.1;
		      }
		  }
	      }
	    else if(Tct[i][j]==5)//epi
	      {
		PinPrePat[i][j]=0;
		PinPrePat[ii][j]=0;

		if(Tmt[i][j]==1)//up, all PIN2
		  {
		  
		    if(j>=yLRCstraight+6*HMZ+6)
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		    else if(j>=yLRCstraight)
		      {
			PinPrePat[i][j]=0.75;
			PinPrePat[ii][j]=0.75;
		      }
		    else
		      {
			PinPrePat[i][j]=0.25;//later automatic from PLT repression?
			PinPrePat[ii][j]=0.25;
		      }
		   
		  }
		else if(Tmt[i][j]==3)//inward
		  {
		   
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
		else if(Tmt[i][j]==4)
		  {
		    if(j<yR)
		      {
			PinPrePat[i][j]=0.1;
			PinPrePat[ii][j]=0.1;
		      }
		  }
		else if(Tmt[i][j]==2)//down, PIN4 mediated
		  {
		     if(j<yLRCstraight)
		       {
			 PinPrePat[i][j]=0.75;
			 PinPrePat[ii][j]=0.75;
		       }
		  }
		
	      }
	    else if(Tct[i][j]==6)//cortex
	      {
		PinPrePat[i][j]=0;
		PinPrePat[ii][j]=0;


		if(CellZonationState[q]>30)
		  {
		    if(Tmt[i][j]==1)//up
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		  }
		else
		  {
		    if(Tmt[i][j]==2)//down //PIN1, PIN4 and PIN2 together?
		      {		
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
			
		      }
		  }
		if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
		if(Tmt[i][j]==4)
		  {
		    if(j<yR)
		      {
			 PinPrePat[i][j]=0.1;
			 PinPrePat[ii][j]=0.1;
		      }
		  }
	      }
	    else if(Tct[i][j]==7)//endodermis
	      {
		PinPrePat[i][j]=0;
		PinPrePat[ii][j]=0;

		if(Tmt[i][j]==2)//down
		  {
		    
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		else if(Tmt[i][j]==3)//in
		  {
		     PinPrePat[i][j]=0.3;
		     PinPrePat[ii][j]=0.3;
		  }
		else if(Tmt[i][j]==4)
		  {
		    if(j<yR)
		      {
			 PinPrePat[i][j]=0.1;
			 PinPrePat[ii][j]=0.1;
		      }
		  }
		
	      }
	    else if(Tct[i][j]>=8 && Tct[i][j]<12)//pericycle vasculature
	      {
		PinPrePat[i][j]=0;
		PinPrePat[ii][j]=0;

		if(Tmt[i][j]==2)//down
		  {
		    if(j<yR)
		      {
			PinPrePat[i][j]=0.5;
			PinPrePat[ii][j]=0.5;
		      }
		    else
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		  }
		if(CellZonationState[q]<=30)
		  {
		    
		    if(Tmt[i][j]==3 || Tmt[i][j]==4)
		      {
			if(j<yLRCstraight)
			  {
			    PinPrePat[i][j]=0.5;
			    PinPrePat[ii][j]=0.5;
			  }
			else if(j<yLRCstraight+10*HMZ+10)
			  {
			    PinPrePat[i][j]=0.25;
			    PinPrePat[ii][j]=0.25;
			  }
			else
			  {
			     PinPrePat[i][j]=0.15;
			     PinPrePat[ii][j]=0.15;
			  }
		      }
		  }
		else
		  {
		    if(Tct[i][j]==8 && Tmt[i][j]==3)
		      {
			PinPrePat[i][j]=0.15;
			PinPrePat[ii][j]=0.15;
		      }
		    else if(Tmt[i][j]==4)
		      {
			PinPrePat[i][j]=0.15;
			PinPrePat[ii][j]=0.15;
		      }
		  }
	      }
	    else if(Tct[i][j]==12)//QC, identical
	      {
		PinPrePat[i][j]=0;
		PinPrePat[ii][j]=0;


		if(j==CellMinJ[Cell[i][j]])//works better than Tmt[i][j]==4
		  {
		    PinPrePat[i][j]=0.5;
		    PinPrePat[ii][j]=0.5;
		  }
	      }
	    else if(Tct[i][j]==13)//columella bottom layer,identical
	      {
		PinPrePat[i][j]=0;
		PinPrePat[ii][j]=0;


		if(Tmt[i][j]<4)
		  {
		    
		    PinPrePat[i][j]=0.5;
		    PinPrePat[ii][j]=0.5;
		  }
	      }
	    else if(Tct[i][j]>13)//columella except bottom layer, identical
	      {
		PinPrePat[i][j]=0;
		PinPrePat[ii][j]=0;

		if(Tct[i][j]<17)//not the topmost one
		  {
		    if(Tmt[i][j]<=3)//left,right and up
		      {
			PinPrePat[i][j]=0.75;
			PinPrePat[ii][j]=0.75;
		      }
		    else 
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		  }
		else
		  {
		    if(Tmt[i][j]==1)
		      {
			PinPrePat[i][j]=0.75;
			PinPrePat[ii][j]=0.75;
		      }
		    else
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		  }
	      }	 
	  }
      }

  */
  
 
  
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)
	  {
	    q=Cell[i][j];
	    ii=n-1-i;

	    if(Tct[i][j]==12)//QC
	      Aux1PrePat[i][j]=2;
	    if(Tct[i][j]>12 && Tct[i][j]!=13)//columella behalve onderste laag
	      Aux1PrePat[i][j]=3;
	    else if(Tct[i][j]==13)
	      Aux1PrePat[i][j]=1;
	    else if (Tct[i][j]>=1 && Tct[i][j]<=4)//lrc 1-4
{
#ifdef AUX1Epi_High
		if(Tct[i][j]==1)
		  Aux1PrePat[i][j]=1*2;
		else
#endif
	      Aux1PrePat[i][j]=1;
}
	    else if(((i<n/2 && j<=(int)(arad1*i+brad1)) || (i>n/2 && j<=(arad1*ii+brad1))) && (Tct[i][j]<8 || Tct[i][j]>12))
	      Aux1PrePat[i][j]=0.5;
	   
	    else if(Tct[i][j]==5)//epi
	      {
	       	if(CellMaxJ[Cell[i][j]]<yMZEZ-16*HMZ-16)
{
#ifdef AUX1Epi_High
		if(Tct[i][j]==5)
		  Aux1PrePat[i][j]=0.1*2;
		else
#endif
		  Aux1PrePat[i][j]=0.1; 
}
		else if(CellMaxJ[Cell[i][j]]<yMZEZ-8*HMZ-8)
{
#ifdef AUX1Epi_High
		if(Tct[i][j]==5)
		  Aux1PrePat[i][j]=0.3*2;
		else
#endif
		  Aux1PrePat[i][j]=0.3; 
}
		else if(CellMaxJ[Cell[i][j]]<yMZEZ-4*HMZ-4)
{
#ifdef AUX1Epi_High
		if(Tct[i][j]==5)
		  Aux1PrePat[i][j]=0.6*2;
		else
#endif
		  Aux1PrePat[i][j]=0.6;
}
		else if(CellMaxJ[Cell[i][j]]<yMZEZ)
		{
#ifdef AUX1Epi_High
		if(Tct[i][j]==5)
		  Aux1PrePat[i][j]=0.8*2;
		else
#endif
		  Aux1PrePat[i][j]=0.8;
		 }
		else
		{
#ifdef AUX1Epi_High
		if(Tct[i][j]==5)
		  Aux1PrePat[i][j]=1*2;
		else
#endif
		  Aux1PrePat[i][j]=1;
		  }
		  	      }
	    else if(Tct[i][j]==6)//cortex
	      {
		if(CellMaxJ[Cell[i][j]]<yMZEZ-4*HMZ-4)
		  Aux1PrePat[i][j]=0.1;//0.8;
		else if(CellMaxJ[Cell[i][j]]<yMZEZ)
		  Aux1PrePat[i][j]=0.2;
		else 
		  Aux1PrePat[i][j]=0.5;
	      }
	    else if(Tct[i][j]>=8 && Tct[i][j]<=11)//peri and vasc
	      {
		if(CellMaxJ[q]<yQC+6*HMZ+6)
		  Aux1PrePat[i][j]=2;
		else if(CellMaxJ[q]<yQC+12*HMZ+12)
		  Aux1PrePat[i][j]=1.5;
		/*else if(CellMaxJ[q]<yLRC1-4*HMZ-4)//
		  Aux1PrePat[i][j]=0.5;
		else if (CellMaxJ[q]<yLRC1+10*HMZ+10)//
		  Aux1PrePat[i][j]=1;
		else
		Aux1PrePat[i][j]=1.5;*/
		else
		{
#ifdef AUX1Vasc_High
		  if(Tct[i][j]>8 && Tct[i][j]<=11)
		  Aux1PrePat[i][j]=1*2;
		  else if(Tct[i][j]==8)
#endif 
		  Aux1PrePat[i][j]=1;
		 }
	      }
	  
	  }
      }
  
 
  
  for(i=0;i<n;i++)
   for(j=0;j<m;j++)
     {
       if(Tct[i][j]>7 && Tct[i][j]!=12)
	 Lax3PrePat[i][j]=1;
       else
	 Lax3PrePat[i][j]=0;
     }
 
  //only PIN1, no reflux at all
  /*
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//membrane
	  {
	    if(!(Tct[i][j]==8 || Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11))
	      {
		PinPrePat[i][j]=0;
		Aux1PrePat[i][j]=0;
	      }
a	    else
	      {
		if(Tmt[i][j]==3 || Tmt[i][j]==4)
		  PinPrePat[i][j]=0;

		if(Tmt[i][j]==2 && CellMaxJ[Cell[i][j]]<yLRCstraight-4*HMZ-4)
		   PinPrePat[i][j]=0.125;
		else if(Tmt[i][j]==2 && CellMaxJ[Cell[i][j]]<yLRCstraight)
		  PinPrePat[i][j]=0.25;
		else if(Tmt[i][j]==2 && CellMaxJ[Cell[i][j]]<yLRCstraight+4*HMZ+4)
		   PinPrePat[i][j]=0.5;
		
	      }
	  }
      }
  */

  //block of lateral transport in OZ
  /*
  for(i=0;i<n;i++)
      for(j=0;j<m;j++)
	{
	  if(j>yLRC1-8*HMZ-8 && j<yLRC1+8*HMZ+8)//OZ
	    {
	      if(Tct[i][j]==6 || Tct[i][j]==7 || Tct[i][j]==8 || Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11)//cortex and endo
		{
		  if(Tmt[i][j]==3 || Tmt[i][j]==4)//membrane
		    {
		      PinPrePat[i][j]=0;
		    }
		}
	      if(Tct[i][j]==5)//lrc & epi
		{
		  if(Tmt[i][j]==3)
		    {
		      PinPrePat[i][j]=0;
		    }
		}
	    }
	  if(j>yLRC1-8*HMZ-8)
	    {
	       if(Tct[i][j]==1)//lrc & epi
		{
		  if(Tmt[i][j]==3)
		    {
		      PinPrePat[i][j]=0;
		    }
		}
	    }
	}
  */

  /*
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	//if(j<yLRC1+15*HMZ+15)
	if(j>yQC)
	  {
	    if(j>yLRC1-15*HMZ-15)//if(j<yLRC1+15*HMZ+15)
	      {
		if(Tmt[i][j]==3 || Tmt[i][j]==4)
		  PinPrePat[i][j]=0;
	      }
	  }
      }
  */

 
#ifdef CYTOKININ
 
  double ttime=time%(3*3600);
  double phase;
  
  
  if(ttime<(1*3600))//2.5
    phase=-1;
  else
    phase=1;
  //dit geeft een cycle van 5 uur met 2.5 uur down 2.5 uur up
  
  
  double factor_AUX1;
  double factor_pAux;
  
  if(phase==-1)
    {
      factor_AUX1=0.8;
      factor_pAux=1;
    }
  else
    {
      factor_AUX1=1;
      factor_pAux=1;//10
    }
#endif
  
  //update membrane protein concentration based on new prepatterns and gene expression levels
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Cell[i][j]!=-1)//grid point inside cell
	  {
	    q=Cell[i][j];
	    
#ifdef AUXINREGULATESPIN
	    
	    //Use [auxin] in cell-wall for feedback 
	    //ii=n-1-i;  
	    if(Tct[i][j]<=5  && Tct[i][j]!=0) //Feedabck in epidermis and lateral root cap
	      {
		if(Tmt[i][j]==1)//up
		  {
		    if(Tc[i][j+1]==0)
		      Cpin1M[i][j]=Pin[q]*PinPrePat[i][j]*(.1+1.5*(((Caux1[i][j+1])*(Caux1[i][j+1]))/((Caux1[i][j+1])*(Caux1[i][j+1])+10*10))); //Km=50
		  }
		else if(Tmt[i][j]==3) //in
		  {
		    if(i<n/2)//left cell
		      {
			if(Tc[i+1][j]==0)
			  Cpin1M[i][j]=Pin[q]*PinPrePat[i][j]*(.1+1.5*(((Caux1[i+1][j])*(Caux1[i+1][j]))/((Caux1[i+1][j])*(Caux1[i+1][j])+10*10)));
		      }									
		    else 
		      {
			if(Tc[i-1][j]==0)
			  Cpin1M[i][j]=Pin[q]*PinPrePat[i][j]*(.1+1.5*(((Caux1[i-1][j])*(Caux1[i-1][j]))/((Caux1[i-1][j])*(Caux1[i-1][j])+10*10)));
		      }
		  }
		else
		  Cpin1M[i][j]=Pin[q]*PinPrePat[i][j];
	      }
	   else
#endif

#ifdef VASCPIN_ONLY	
	  if(Tct[i][j]<7 && j>yLRCstraight)
	     Cpin1M[i][j]=Pin[q]*PinPrePat[i][j] *0;
	  else
#endif
#ifdef VASCPIN_PIN2UP
	  if(Tmt[i][j]>2 && Tct[i][j]<7 && j>yLRCstraight)
	     Cpin1M[i][j]=Pin[q]*PinPrePat[i][j] *0;
	  else 	  
#endif
#ifdef VASCPIN_PIN2LOW
	  if(Tct[i][j]<7 && j>yLRCstraight)
	     Cpin1M[i][j]=Pin[q]*PinPrePat[i][j] *0.5;
	  else
#endif
	    Cpin1M[i][j]=Pin[q]*PinPrePat[i][j];
	    
#ifdef CYTOKININ
	    if((Tct[i][j]==5 || Tct[i][j]==1 || Tct[i][j]==8 || Tct[i][j]==9) && (j>=yOZs && j<=yOZe))//THIS SHOULD BE CHANGED TO CELL BEIGN PART OF OZ?
	      Aux1M[i][j]=Aux1[Cell[i][j]]*Aux1PrePat[i][j]*factor_AUX1;
	    else
#endif
	    Aux1M[i][j]=Aux1[Cell[i][j]]*Aux1PrePat[i][j];	  
	    Lax3M[i][j]=Lax3[Cell[i][j]]*Lax3PrePat[i][j];
	  }
	else
	  {
	    Cpin1M[i][j]=0;
	    Aux1M[i][j]=0;
	    Lax3M[i][j]=0; 	    
	  }   	
      }
  
  
  //update auxin fluxes and metabolism based on new tissue architecture and gene expression levels
  //auxin prod, decay, diff, flux rates:
  //direction 0 i+1;
  //direction 1 i-1;
  //direction 2 j+1;
  //direction 3 j-1;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	daux[i][j]=0;//default no decay
	paux[i][j]=0;//default no production
	for(k=0;k<4;k++)
	  {	  
	    Inf[i][j][k]=0;
	    Eff[i][j][k]=0;
	    DiffAuxC[i][j][k]=0;
	  }
      }
 

  //daux depends on Gh33 gene expression
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	q=Cell[i][j];
	if(q!=-1)
	  daux[i][j]=(1-gh33fac)*dAux+gh33fac*0.01*Gh33[q]*dAux;
      }
  //paux depends on Yuc3 expression
  double volfactor;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
	{
	  ii=n-1-i;
	  q=Cell[i][j];
	  if(q!=-1)
	    { 
	      //still ghost points at yR and yEND out of thin air?
	      if(Tct[i][j]==12 || Tct[i][j]==17)//QC and columella layer 5: okay
	      {
		volfactor=50;
#ifdef LOWAUXINPROD_QC //publication
		volfactor=50*0.01;
#endif
		}
	      else if(j<yQC+2*HMZ+2 && (Tct[i][j]==8|| Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11))//this part I changed
		{
		volfactor=50;
#ifdef LOWAUXINPROD_QC //publication
		volfactor=50*0.01;
#endif
	}
	      else if((j<yQC+HMZ+1 && (j<(int)(arad3*i+brad3) && j<(int)(arad3*ii+brad3))) && (Tct[i][j]==5 || Tct[i][j]==6 || Tct[i][j]==7 || Tct[i][j]==8))//epi-peri cells next to QC, added j<yQC+HMZ+1 to delimit it to lowest peri cell            	
		{
		volfactor=50;
#ifdef LOWAUXINPROD_QC //publication
		volfactor=50*0.01;
#endif
		}
	      else if(Tct[i][j]>=1 && Tct[i][j]<=4)//Lateral Root Cap
		{
		volfactor=20*(int)(nrcellsstraightMZ/20);//15;
#ifdef LOWAUXINPROD_LRC //publication
		volfactor=20*(int)(nrcellsstraightMZ/20)*0.01;//15;
#endif
	}
	      else if(Tct[i][j] > 12 && Tct[i][j]< 17 )//Columella layers 1-4 from the bottom
		volfactor = 25;
		  else if(j<yLRC1-(int)(.25*nrcellsstraightMZ))
		volfactor = 3;
	      else 
		volfactor=1;
	      
	      paux[i][j]=volfactor*2*((1-yuc3fac)*pAux+yuc3fac*0.01*Yuc3[q]*pAux)*HMZ/CellHeight[q];
	    }
	}


#ifdef AUXINAPPLICATION
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(i==0 || i==n-1 || j==0)
	  paux[i][j]+=1.00/xstep;
      }
#endif


//#define POINTYROOTTIP
//#define SEMI3D

//#define PHDROP

#ifndef SATURATION
  //Set Inf from Lax3,Lax2,Aux1
  double r,rmin;
  double v=1;
  double vf=1;//0.5;//0.98;//1.0;
  double inf2=inf;

  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)
	  {
	    //nearest cell:
	    if(i!=0)
	      if(Tc[i-1][j]==2)
		{
#ifdef  SEMI3D
		  if(i>xm)//right half, than you radius bigger than in i-1
		    {
		      if(i>n-1-(disttoepi+totsizeepi+totsizecort+totsizeendo))
			{
			  r=abs(i-xm);
#ifdef POINTYROOTTIP
			  if(j<yLRCstraight)
			    vf=0.98;
			  else if(j<yLRC1)
			    vf=0.98+0.02*(j-yLRCstraight)/(yLRC1-yLRCstraight);
			  else
			  vf=1;
#endif
			  rmin=r-vf;
			  v=rmin/r; 
			}
		      else
			v=1;
		    }
		  else
		    v=1;
#endif
#ifdef PHDROP
		  if(j>=yLRCstraight-5*HMZ-5 && j<yLRCstraight+8*HEZ+8)
		    inf2=2*inf;
		  else
		    inf2=0.25*inf;
#endif
		  Inf[i][j][1]=v*(-inf2-aux1fac*Aux1M[i-1][j]-aux1fac*Lax3M[i-1][j])*invspstep;
		}
	    if(i!=n-1)	
	      if(Tc[i+1][j]==2)
		{
#ifdef  SEMI3D
		  if(i<xm)//left half, than your radius bigger than in i+1
		    {
		      if(i<disttoepi+totsizeepi+totsizecort+totsizeendo)
			{
			  r=abs(i-xm);
#ifdef POINTYROOTTIP
			  if(j<yLRCstraight)
			    vf=0.98;
			  else if(j<yLRC1)
			    vf=0.98+0.02*(j-yLRCstraight)/(yLRC1-yLRCstraight);
			  else
			  vf=1;
#endif
			  rmin=r-vf;
			  v=rmin/r;
			}
		      else 
			v=1;
		    }
		  else
		    v=1;
#endif
#ifdef PHDROP
		  if(j>=yLRCstraight-5*HMZ-5 && j<yLRCstraight+8*HEZ+8)
		    inf2=2*inf;
		  else
		    inf2=0.25*inf;
#endif
		  Inf[i][j][0]=v*(-inf2-aux1fac*Aux1M[i+1][j]-aux1fac*Lax3M[i+1][j])*invspstep;
		}
	    if(j!=0)
	      if(Tc[i][j-1]==2)
		{
#ifdef PHDROP
		  if((j-1)>=yLRCstraight-5*HMZ-5 && (j-1)<yLRCstraight+8*HEZ+8)
		    inf2=2*inf;
		  else
		    inf2=0.25*inf;
#endif
		  Inf[i][j][3]=(-inf2-aux1fac*Aux1M[i][j-1]-aux1fac*Lax3M[i][j-1])*invspstep;
		}
	    if(j!=m-1)
	      if(Tc[i][j+1]==2)
		{
#ifdef PHDROP
		  if((j+1)>=yLRCstraight-5*HMZ-5 && (j+1)<yLRCstraight+8*HEZ+8)
		    inf2=2*inf;
		  else
		    inf2=0.25*inf;
#endif
		  Inf[i][j][2]=(-inf2-aux1fac*Aux1M[i][j+1]-aux1fac*Lax3M[i][j+1])*invspstep;
		}
	  }
	else if(Tc[i][j]==2)
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		{
#ifdef  SEMI3D
		  if(i<xm)//left half, than your radius bigger than in i+1
		    {
		      if(i<disttoepi+totsizeepi+totsizecort+totsizeendo)
			{
			  r=abs(i-xm);
#ifdef POINTYROOTTIP
			  if(j<yLRCstraight)
			    vf=0.98;
			  else if(j<yLRC1)
			    vf=0.98+0.02*(j-yLRCstraight)/(yLRC1-yLRCstraight);
			  else
			    vf=1;
#endif
			  rmin=r-vf;
			  v=rmin/r;
			}
		      else 
			v=1;
		    }
		  else
		    v=1;
#endif
#ifdef PHDROP
		  if(j>=yLRCstraight-5*HMZ-5 && j<yLRCstraight+8*HEZ+8)
		    inf2=2*inf;
		  else
		    inf2=0.25*inf;
#endif
		  Inf[i][j][0]=v*(inf2+aux1fac*Aux1M[i][j]+aux1fac*Lax3M[i][j])*invspstep;
		}
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		{
#ifdef SEMI3D
		  if(i>xm)//right half, than you radius bigger than in i-1
		    {
		      if(i>n-1-(disttoepi+totsizeepi+totsizecort+totsizeendo))
			{
			  r=abs(i-xm);
#ifdef POINTYROOTTIP
			  if(j<yLRCstraight)
			    vf=0.98;
			  else if(j<yLRC1)
			    vf=0.98+0.02*(j-yLRCstraight)/(yLRC1-yLRCstraight);
			  else 
			    vf=1;
#endif
			  rmin=r-vf;
			  v=rmin/r; 
			}
		      else
			v=1;
		    }
		  else
		    v=1;
#endif
#ifdef PHDROP
		  if(j>=yLRCstraight-5*HMZ-5 && j<yLRCstraight+8*HEZ+8)
		    inf2=2*inf;
		  else
		    inf2=0.25*inf;
#endif
		  Inf[i][j][1]=v*(inf2+aux1fac*Aux1M[i][j]+aux1fac*Lax3M[i][j])*invspstep;
		}
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		{
#ifdef PHDROP
		  if(j>=yLRCstraight-5*HMZ-5 && j<yLRCstraight+8*HEZ+8)
		    inf2=2*inf;
		  else
		    inf2=0.25*inf;
#endif
		  Inf[i][j][2]=(inf2+aux1fac*Aux1M[i][j]+aux1fac*Lax3M[i][j])*invspstep;
		}
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		{
#ifdef PHDROP
		  if(j>=yLRCstraight-5*HMZ-5 && j<yLRCstraight+8*HEZ+8)
		    inf2=2*inf;
		  else
		    inf2=0.25*inf;
#endif
		  Inf[i][j][3]=(inf2+aux1fac*Aux1M[i][j]+aux1fac*Lax3M[i][j])*invspstep;
		}
	  }
      }
  //Set Eff from Tc
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)//wall, neighboring membrane
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==2)
		{	
#ifdef SEMI3D
		  if(i<xm)//left half, than your radius bigger than in i+1
		    {
		      if(i<disttoepi+totsizeepi+totsizecort+totsizeendo)
			{
			  r=abs(i-xm);
#ifdef POINTYROOTTIP
			  if(j<yLRCstraight)
			    vf=0.98;
			  else if(j<yLRC1)
			    vf=0.98+0.02*(j-yLRCstraight)/(yLRC1-yLRCstraight);
			  else
			    vf=1;
#endif
			  rmin=r-vf;
			  v=rmin/r;
			}
		      else v=1;
		    }
		  else
		    v=1;
#endif	  
		  Eff[i][j][0]=v*(eff*invspstep+pPUMP*(pinfac1*Cpin1M[i+1][j]+(1-pinfac1)*100*PinPrePat[i+1][j])*invspstep);
		}
	    if(i!=0)
	      if(Tc[i-1][j]==2)
		{
#ifdef  SEMI3D
		   if(i>xm)//right half, than you radius bigger than in i-1
		    {
		      if(i>n-1-(disttoepi+totsizeepi+totsizecort+totsizeendo))
			{
			  r=abs(i-xm);
#ifdef POINTYROOTTIP
			  if(j<yLRCstraight)
			    vf=0.98;
			  else if(j<yLRC1)
			    vf=0.98+0.02*(j-yLRCstraight)/(yLRC1-yLRCstraight);
			  else
			    vf=1;
#endif
			  rmin=r-vf;
			  v=rmin/r; 
			}
		      else
			v=1;
		    }
		  else
		    v=1;
#endif
		   Eff[i][j][1]=v*(eff*invspstep+pPUMP*(pinfac1*Cpin1M[i-1][j]+(1-pinfac1)*100*PinPrePat[i-1][j])*invspstep);

		}
	    if(j!=m-1)
	      if(Tc[i][j+1]==2)
		Eff[i][j][2]=eff*invspstep+pPUMP*(pinfac1*Cpin1M[i][j+1]+(1-pinfac1)*100*PinPrePat[i][j+1])*invspstep;;
	    if(j!=0)
	      if(Tc[i][j-1]==2)
		Eff[i][j][3]=eff*invspstep+pPUMP*(pinfac1*Cpin1M[i][j-1]+(1-pinfac1)*100*PinPrePat[i][j-1])*invspstep;;
	  }
	else if(Tc[i][j]==2)//membrane, neighboring wall
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		{
#ifdef  SEMI3D
		  if(i<xm)//left half, than your radius bigger than in i+1
		    {
		      if(i<disttoepi+totsizeepi+totsizecort+totsizeendo)
			{
			  r=abs(i-xm);
#ifdef POINTYROOTTIP
			  if(j<yLRCstraight)
			    vf=0.98;
			  else if(j<yLRC1)
			    vf=0.98+0.02*(j-yLRCstraight)/(yLRC1-yLRCstraight);
			  else
			    vf=1;
#endif
			  rmin=r-vf;
			  v=rmin/r;
			}
		      else v=1;
		    }
		  else
		    v=1;
#endif  
		  Eff[i][j][0]=v*(-eff*invspstep-pPUMP*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep);
		}
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		{
#ifdef  SEMI3D
		  if(i>xm)//right half, than you radius bigger than in i-1
		    {
		      if(i>n-1-(disttoepi+totsizeepi+totsizecort+totsizeendo))
			{
			  r=abs(i-xm);
#ifdef POINTYROOTTIP
			  if(j<yLRCstraight)
			    vf=0.98;
			  else if(j<yLRC1)
			    vf=0.98+0.02*(j-yLRCstraight)/(yLRC1-yLRCstraight);
			  else
			    vf=1;
#endif
			  rmin=r-vf;
			  v=rmin/r; 
			}
		      else
			v=1;
		    }
		  else
		    v=1;
#endif
		  Eff[i][j][1]=v*(-eff*invspstep-pPUMP*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep);
		}
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		Eff[i][j][2]=-eff*invspstep-pPUMP*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		Eff[i][j][3]=-eff*invspstep-pPUMP*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
	  }
      }
#endif

  //Set DiffAuxC from Tc
  double DauxCC=0.5*DauxC*tstep/(xstep*xstep);
  double DauxWW=0.5*DauxW*tstep/(xstep*xstep);
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)//wall grid point: only diffuse towards other wall grid point
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		{
#ifdef  SEMI3D
		  if(i<xm)//left half, than your radius bigger than in i+1
		    {
		      if(i<disttoepi+totsizeepi+totsizecort+totsizeendo)
		      {
			  r=abs(i-xm);
#ifdef POINTYROOTTIP
			  if(j<yLRCstraight)
			    vf=0.98;
			  else if(j<yLRC1)
			    vf=0.98+0.02*(j-yLRCstraight)/(yLRC1-yLRCstraight);
			  else
			    vf=1;
#endif
			  rmin=r-vf;
			  v=rmin/r;
			}
		      else v=1;
		    }
		  else
		    v=1;
#endif
		  DiffAuxC[i][j][0]=v*DauxWW;
		}
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		{
#ifdef  SEMI3D
		  if(i>xm)//right half, than you radius bigger than in i-1
		    {
		      if(i>n-1-(disttoepi+totsizeepi+totsizecort+totsizeendo))
			{
			  r=abs(i-xm);
#ifdef POINTYROOTTIP
			  if(j<yLRCstraight)
			    vf=0.98;
			  else if(j<yLRC1)
			    vf=0.98+0.02*(j-yLRCstraight)/(yLRC1-yLRCstraight);
			  else
			    vf=1;
#endif
			  rmin=r-vf;
			  v=rmin/r; 
			}
		      else
			v=1;
		    }
		  else
		    v=1;
#endif
		  DiffAuxC[i][j][1]=v*DauxWW;
		}
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		DiffAuxC[i][j][2]=DauxWW;
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		DiffAuxC[i][j][3]=DauxWW;
	  }
	else if(Tc[i][j]==1 || Tc[i][j]==2)//cell grid point: only diffuse towards other cell grid point
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==1 || Tc[i+1][j]==2)
		{
#ifdef  SEMI3D
		  if(i<xm)//left half, than your radius bigger than in i+1
		    {
		      if(i<disttoepi+totsizeepi+totsizecort+totsizeendo)
			{
			   r=abs(i-xm);
#ifdef POINTYROOTTIP
			   if(j<yLRCstraight)
			    vf=0.98;
			   else if(j<yLRC1)
			    vf=0.98+0.02*(j-yLRCstraight)/(yLRC1-yLRCstraight);
			   else
			     vf=1;
#endif
			   rmin=r-vf;
			   v=rmin/r;
			}
		      else
			v=1;
		    }
		  else
		    v=1;
#endif
		  DiffAuxC[i][j][0]=v*DauxCC;
		}
	    if(i!=0)
	      if(Tc[i-1][j]==1 || Tc[i-1][j]==2)
		{
#ifdef  SEMI3D
		  if(i>xm)//right half, than you radius bigger than in i-1
		    {
		      if(i>n-1-(disttoepi+totsizeepi+totsizecort+totsizeendo))
			{
			  r=abs(i-xm);
#ifdef POINTYROOTTIP
			  if(j<yLRCstraight)
			    vf=0.98;
			  else if(j<yLRC1)
			    vf=0.98+0.02*(j-yLRCstraight)/(yLRC1-yLRCstraight);
			  else
			  vf=1;
#endif
			  rmin=r-vf;
			  v=rmin/r; 
			}
		      else
			v=1;
		    }
		  else
		    v=1;
#endif
		  DiffAuxC[i][j][1]=v*DauxCC;
		}
	    if(j!=m-1)
	      if(Tc[i][j+1]==1 || Tc[i][j+1]==2)
		DiffAuxC[i][j][2]=DauxCC;
	    if(j!=0)
	      if(Tc[i][j-1]==1 || Tc[i][j-1]==2)
		DiffAuxC[i][j][3]=DauxCC;
	  }
      }


  //Now we need to adjust for getting a normal influx/efflux relation between root and shoot
#ifdef NORMALSHOOTROOTCONNECTION
  double auxinvirtlayer=0.8*0.5;//1;//0.5;//10;//0.5
#endif
#ifdef SHOOTROOTCUT
  double auxinvirtlayer=0.8*0.5*exp(-0.003*time);//0;
#endif
#ifdef LOWAUXIN_FROMSHOOT 
#ifndef LOW_AUXINFROMSHOOT //for publication
 double auxinvirtlayer=2*1.5*0.8*0.5*0.8;
#else
  double auxinvirtlayer=2*1.5*0.8*0.5*0.8*0.6667; //reduction
#endif
#endif
  int jbot;
  int jstore;

  for(i=0;i<n;i++)
    for(j=m-1;j>=0;j--)
      {
	
	if(Tc[i][j]==0 && Tc[i][j-1]==2)//wall above cell
	  {
	    if(Tct[i][j-1]==5 || Tct[i][j-1]==6 || Tct[i][j-1]==7)//efflux to shoot from epi/cort/endo
	      {		  
		jbot=CellMinJ[Cell[i][j-1]]-1;
		daux[i][j]=-0.1*Inf[i][jbot][2];
	      }
	    else if(Tct[i][j-1]==8 || Tct[i][j-1]==9)//influx from shoot to pericycle and vasculature
	      {
		jbot=CellMinJ[Cell[i][j-1]]-1;
		paux[i][j]=(Eff[i][jbot][2])*auxinvirtlayer;
	      }
	    break;
	  }
      }
}

