#include "Header.hh"


void dsolvetridiag(int ni)
{
  /**
   * ni - number of equations
   * a - sub-diagonal (means it is the diagonal below the main diagonal) -- indexed from 1..n-1
   * b - the main diagonal
   * c - sup-diagonal (means it is the diagonal above the main diagonal) -- indexed from 0..n
   * rhs - right part
   * x - the answer
   */
  int i;
  double mm;
  
  for (i = 1; i < ni; i++)
    {
      mm = a[i]/b[i-1];
      b[i] = b[i] - mm*c[i-1];
      rhs[i] = rhs[i] - mm*rhs[i-1];
    }

  res[ni-1] = rhs[ni-1]/b[ni-1];
  
  for (i = ni - 2; i >= 0; i--)
    res[i]=(rhs[i]-c[i]*res[i+1])/b[i];
  
  for(i=0;i<ni;i++)
    rhs[i]=res[i];
}


void UpdateAuxin()
{

  int i,j;
  static int Direction=0;
  int q;
  double func;
#ifndef SATURATION  
  if(Direction==0)//Implicit integration in j direction
    {             //Explicit integration in i direction
      Direction=1;
      
      //do one direction implicit one explicit
      for(i=0;i<n;i++)
	{
	  for(j=0;j<m;j++)
	    {
	      //Compute coefficients for implicit diffusion and fluxes
	      //diffusion only

	      if(Tc[i][j]==3)
		{
		  a[j]=0;
		  c[j]=0;
		  b[j]=1;
		  rhs[j]=0;
		}
	      else   
		{
		  a[j]=-DiffAuxC[i][j][3];//j-1
		  c[j]=-DiffAuxC[i][j][2];//j+1 
		  b[j]=1-(a[j]+c[j]);//j 
		  if(Tc[i][j]==0)
		    {
		      //influx
		      b[j]+=-0.5*tstep*(Inf[i][j][2]+Inf[i][j][3]);
		      //efflux
		      a[j]+=-0.5*tstep*Eff[i][j][3];
		      c[j]+=-0.5*tstep*Eff[i][j][2];
		    }
		  else if(Tc[i][j]==2)
		    {
		      //influx
		      a[j]+=-0.5*tstep*Inf[i][j][3];
		      c[j]+=-0.5*tstep*Inf[i][j][2];
		      //efflux
		      b[j]+=-0.5*tstep*(Eff[i][j][2]+Eff[i][j][3]);
		    }
		  
		  //explicit diffusion and fluxes
		  rhs[j]=Caux1[i][j]+DiffAuxC[i][j][0]*(Caux1[i+1][j]-Caux1[i][j])
		    +DiffAuxC[i][j][1]*(Caux1[i-1][j]-Caux1[i][j]);
		  if(Tc[i][j]==0)
		    {
		      rhs[j]+=0.5*tstep*(Eff[i][j][0]*Caux1[i+1][j]+Eff[i][j][1]*Caux1[i-1][j]);
		      rhs[j]+=0.5*tstep*((Inf[i][j][0]+Inf[i][j][1])*Caux1[i][j]);
		    }
		  else if(Tc[i][j]==2)
		    {
		      rhs[j]+=0.5*tstep*((Eff[i][j][0]+Eff[i][j][1])*Caux1[i][j]);
		      rhs[j]+=0.5*tstep*(Inf[i][j][0]*Caux1[i+1][j]+Inf[i][j][1]*Caux1[i-1][j]); 
		    }
		}//else: not Tc[i][j]==3
	    }
	  //Compute implicit diffusion and use this to update rhs
	  //Use new rhs to update volt2
	  dsolvetridiag(m); 
	  for(j=0;j<m;j++)
	    {
	      if(Tc[i][j]!=3)
		{
		  func=0.5*tstep*(paux[i][j]-daux[i][j]*Caux1[i][j]);
		  Cauxint[i][j]=rhs[j]+func;
		}
	    }
	}
      
    }
  if(Direction==1)//Implicit integration in i direction
    {             //Explicit integration in j direction
      Direction=0;
      
      //do one direction implicit one explicit
      for(j=0;j<m;j++)
	{
	  for(i=0;i<n;i++)
	    {
	      //Compute coefficients for implicit diffusion and fluxes
	      //diffusion only
	      if(Tc[i][j]==3)
		{
		  a[i]=0;
		  c[i]=0;
		  b[i]=1;
		  rhs[i]=0;
		}
	      else
		{
		  a[i]=-DiffAuxC[i][j][1];//i-1 direction
		  c[i]=-DiffAuxC[i][j][0];//i+1 direction
		  b[i]=1-(a[i]+c[i]);//i;
		  if(Tc[i][j]==0)
		    {
		      //influx
		      b[i]+=-0.5*tstep*(Inf[i][j][0]+Inf[i][j][1]);
		      //efflux
		      a[i]+=-0.5*tstep*Eff[i][j][1];
		      c[i]+=-0.5*tstep*Eff[i][j][0];
		    }
		  else if(Tc[i][j]==2)
		    {
		      //influx
		      a[i]+=-0.5*tstep*Inf[i][j][1];
		      c[i]+=-0.5*tstep*Inf[i][j][0];
		      //efflux
		      b[i]+=-0.5*tstep*(Eff[i][j][0]+Eff[i][j][1]);
		    }
		  
		  
		  //explicit diffusion and fluxes
		  rhs[i]=Cauxint[i][j]+DiffAuxC[i][j][2]*(Cauxint[i][j+1]-Cauxint[i][j])
		    +DiffAuxC[i][j][3]*(Cauxint[i][j-1]-Cauxint[i][j]);
		  if(Tc[i][j]==0)
		    {
		      rhs[i]+=0.5*tstep*(Eff[i][j][2]*Cauxint[i][j+1]+Eff[i][j][3]*Cauxint[i][j-1]);
		      rhs[i]+=0.5*tstep*((Inf[i][j][2]+Inf[i][j][3])*Cauxint[i][j]);
		    }
		  else if(Tc[i][j]==2)
		    {
		      rhs[i]+=0.5*tstep*((Eff[i][j][2]+Eff[i][j][3])*Cauxint[i][j]);
		      rhs[i]+=0.5*tstep*(Inf[i][j][2]*Cauxint[i][j+1]+Inf[i][j][3]*Cauxint[i][j-1]);
		    }
		}
	    }
	  //Compute implicit diffusion and use this to update rhs
	  //Use new rhs to update volt2
	  dsolvetridiag(n); 
	  for(i=0;i<n;i++)
	    { 
	      if(Tc[i][j]!=3)
		{
		  func=0.5*tstep*(paux[i][j]-daux[i][j]*Cauxint[i][j]);
		  Caux2[i][j]=rhs[i]+func;
		}
	    }
	}
    }

  //bereken de gemiddelde auxine conc per cell
  for(q=0;q<totcells;q++)
    MeanCells[q]=0;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Cell[i][j]!=-1)
	  {
	    q=Cell[i][j];
	    MeanCells[q]+=Caux1[i][j];
	  }
      }
  for(q=0;q<totcells;q++)
    MeanCells[q]=MeanCells[q]*InvCellSizes[q];
  
  //final update for next timestep
  //doe dit pas na het berekenen van het 
  //gemiddelde auxin level per cell
  //hierdoor doe je genexpressie update
  //nog op basis van oude aux1, net als
  //fluxes, diffusie, decay etc.    
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]!=3)
	  Caux1[i][j]=Caux2[i][j];
      }

#else 

double invspstep=1.0/(double)(xstep);


	  if(Direction==0)//Implicit integration in j direction
    {             //Explicit integration in i direction
      Direction=1;


		for(i=0;i<n;i++)
			for(j=0;j<m;j++)
			  {
#ifndef SATINFLUX
				inf_sat[i][j] = inf;
#else
				//if(Tc[i][j]==0)
					inf_sat[i][j] = inf/(Caux1[i][j]+SatInf);
				//else
					//inf_sat[i][j] = 0;
					
#endif 
#ifndef SATAUXLAX
				aux1fac_sat[i][j] = aux1fac;
#else
				//if(Tc[i][j]==0)
					aux1fac_sat[i][j] = aux1fac/(Caux1[i][j]+SatAux1);
				//else
					//aux1fac_sat[i][j]= 0;
					
#endif
				if(Tc[i][j]==0)
				  {
					//nearest cell:
					if(i!=0)
					  if(Tc[i-1][j]==2)
					{
					  Inf[i][j][1]=(-inf_sat[i][j]-aux1fac_sat[i][j]*Aux1M[i-1][j])*invspstep;
					}
					if(i!=n-1)	
					  if(Tc[i+1][j]==2)
					{
					  Inf[i][j][0]=(-inf_sat[i][j]-aux1fac_sat[i][j]*Aux1M[i+1][j])*invspstep;
					}
					if(j!=0)
					  if(Tc[i][j-1]==2)
					{
					  Inf[i][j][3]=(-inf_sat[i][j]-aux1fac_sat[i][j]*Aux1M[i][j-1])*invspstep;
					}
					if(j!=m-1)
					  if(Tc[i][j+1]==2)
					{
					  Inf[i][j][2]=(-inf_sat[i][j]-aux1fac_sat[i][j]*Aux1M[i][j+1])*invspstep;
					}
				  }
				else if(Tc[i][j]==2)
				  {
					if(i!=n-1)
					  if(Tc[i+1][j]==0)
					{
					  Inf[i][j][0]=(inf_sat[i+1][j]+aux1fac_sat[i+1][j]*Aux1M[i][j])*invspstep;
					}
					if(i!=0)
					  if(Tc[i-1][j]==0)
					{
					  Inf[i][j][1]=(inf_sat[i-1][j]+aux1fac_sat[i-1][j]*Aux1M[i][j])*invspstep;
					}
					if(j!=m-1)
					  if(Tc[i][j+1]==0)
					{
					  Inf[i][j][2]=(inf_sat[i][j+1]+aux1fac_sat[i][j+1]*Aux1M[i][j])*invspstep;
					}
					if(j!=0)
					  if(Tc[i][j-1]==0)
					{
					  Inf[i][j][3]=(inf_sat[i][j-1]+aux1fac_sat[i][j-1]*Aux1M[i][j])*invspstep;
					}
				  }
				  
			  }
          for(i=0;i<n;i++)
			for(j=0;j<m;j++)
			  {  
		
#ifndef SATEFFLUX
					eff_sat[i][j] = eff;
#else
					
					//if(Tc[i][j]==2)
						eff_sat[i][j] = eff/(Caux1[i][j]+SatEff);
					//else
						//eff_sat[i][j] = 0;
#endif

#ifndef SATPIN
					pPUMP_sat[i][j] = pPUMP;
#else
				
					//if(Tc[i][j]==2)
						pPUMP_sat[i][j] = pPUMP/(Caux1[i][j]+SatPin);
					//else
						//pPUMP_sat[i][j]= 0;
#endif
		}
		for(i=0;i<n;i++)
			for(j=0;j<m;j++)
			  {
					if(Tc[i][j]==0)//wall, neighboring membrane
					  {
						if(i!=n-1)
						  if(Tc[i+1][j]==2)
							Eff[i][j][0]=eff_sat[i+1][j]*invspstep+pPUMP_sat[i+1][j]*(pinfac1*Cpin1M[i+1][j]+(1-pinfac1)*100*PinPrePat[i+1][j])*invspstep;;
						if(i!=0)
						  if(Tc[i-1][j]==2)
							Eff[i][j][1]=eff_sat[i-1][j]*invspstep+pPUMP_sat[i-1][j]*(pinfac1*Cpin1M[i-1][j]+(1-pinfac1)*100*PinPrePat[i-1][j])*invspstep;;
						if(j!=m-1)
						  if(Tc[i][j+1]==2)
							Eff[i][j][2]=eff_sat[i][j+1]*invspstep+pPUMP_sat[i][j+1]*(pinfac1*Cpin1M[i][j+1]+(1-pinfac1)*100*PinPrePat[i][j+1])*invspstep;;
						if(j!=0)
						  if(Tc[i][j-1]==2)
							Eff[i][j][3]=eff_sat[i][j-1]*invspstep+pPUMP_sat[i][j-1]*(pinfac1*Cpin1M[i][j-1]+(1-pinfac1)*100*PinPrePat[i][j-1])*invspstep;;
					  }
					else if(Tc[i][j]==2)//membrane, neighboring wall
					  {
						if(i!=n-1)
						  if(Tc[i+1][j]==0)
							Eff[i][j][0]=-eff_sat[i][j]*invspstep-pPUMP_sat[i][j]*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
						if(i!=0)
						  if(Tc[i-1][j]==0)
							Eff[i][j][1]=-eff_sat[i][j]*invspstep-pPUMP_sat[i][j]*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
						if(j!=m-1)
						  if(Tc[i][j+1]==0)
							Eff[i][j][2]=-eff_sat[i][j]*invspstep-pPUMP_sat[i][j]*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
						if(j!=0)
						  if(Tc[i][j-1]==0)
							Eff[i][j][3]=-eff_sat[i][j]*invspstep-pPUMP_sat[i][j]*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
					  }
				}
	 //do one direction implicit one explicit
      for(i=0;i<n;i++)
	{
	  for(j=0;j<m;j++)
	    {
	      //Compute coefficients for implicit diffusion and fluxes
	      //diffusion only

	      if(Tc[i][j]==3)
		{
		  a[j]=0;
		  c[j]=0;
		  b[j]=1;
		  rhs[j]=0;
		}
	      else   
		{
		  a[j]=-DiffAuxC[i][j][3];//j-1
		  c[j]=-DiffAuxC[i][j][2];//j+1 
		  b[j]=1-(a[j]+c[j]);//j 
		  if(Tc[i][j]==0)
		    {
		      //influx
		      b[j]+=-0.5*tstep*(Inf[i][j][2]+Inf[i][j][3]);
		      //efflux
		      a[j]+=-0.5*tstep*Eff[i][j][3];
		      c[j]+=-0.5*tstep*Eff[i][j][2];
		    }
		  else if(Tc[i][j]==2)
		    {
		      //influx
		      a[j]+=-0.5*tstep*Inf[i][j][3];
		      c[j]+=-0.5*tstep*Inf[i][j][2];
		      //efflux
		      b[j]+=-0.5*tstep*(Eff[i][j][2]+Eff[i][j][3]);
		    }
		  
		  //explicit diffusion and fluxes
		  rhs[j]=Caux1[i][j]+DiffAuxC[i][j][0]*(Caux1[i+1][j]-Caux1[i][j])
		    +DiffAuxC[i][j][1]*(Caux1[i-1][j]-Caux1[i][j]);
		  if(Tc[i][j]==0)
		    {
		      rhs[j]+=0.5*tstep*(Eff[i][j][0]*Caux1[i+1][j]+Eff[i][j][1]*Caux1[i-1][j]);
		      rhs[j]+=0.5*tstep*((Inf[i][j][0]+Inf[i][j][1])*Caux1[i][j]);
		    }
		  else if(Tc[i][j]==2)
		    {
		      rhs[j]+=0.5*tstep*((Eff[i][j][0]+Eff[i][j][1])*Caux1[i][j]);
		      rhs[j]+=0.5*tstep*(Inf[i][j][0]*Caux1[i+1][j]+Inf[i][j][1]*Caux1[i-1][j]); 
		    }
		}//else: not Tc[i][j]==3
	    }
	  //Compute implicit diffusion and use this to update rhs
	  //Use new rhs to update volt2
	  dsolvetridiag(m); 
	  for(j=0;j<m;j++)
	    {
	      if(Tc[i][j]!=3)
		{
		  func=0.5*tstep*(paux[i][j]-daux[i][j]*Caux1[i][j]);
		  Cauxint[i][j]=rhs[j]+func;
		}
	    }
	}
      
    }
if(Direction==1)//Implicit integration in i direction
    {             //Explicit integration in j direction
      Direction=0;

		for(i=0;i<n;i++)
			for(j=0;j<m;j++)
			  {
#ifndef SATINFLUX
					inf_sat[i][j] = inf;
#else
					//if(Tc[i][j]==0)
						inf_sat[i][j] = inf/(Cauxint[i][j]+SatInf);
					//else
						//inf_sat[i][j] = 0;
				
#endif 
#ifndef SATAUXLAX
					aux1fac_sat[i][j] = aux1fac;
#else
					//if(Tc[i][j]==0)
						aux1fac_sat[i][j] = aux1fac/(Cauxint[i][j]+SatAux1);
					//else
						//aux1fac_sat[i][j] = 0;
					
#endif
			}
			
		for(i=0;i<n;i++)
			for(j=0;j<m;j++)
			  {
			
				if(Tc[i][j]==0)
				  {
					//nearest cell:
					if(i!=0)
					  if(Tc[i-1][j]==2)
					{
					  Inf[i][j][1]=(-inf_sat[i][j]-aux1fac_sat[i][j]*Aux1M[i-1][j])*invspstep;
					}
					if(i!=n-1)	
					  if(Tc[i+1][j]==2)
					{
					  Inf[i][j][0]=(-inf_sat[i][j]-aux1fac_sat[i][j]*Aux1M[i+1][j])*invspstep;
					}
					if(j!=0)
					  if(Tc[i][j-1]==2)
					{
					  Inf[i][j][3]=(-inf_sat[i][j]-aux1fac_sat[i][j]*Aux1M[i][j-1])*invspstep;
					}
					if(j!=m-1)
					  if(Tc[i][j+1]==2)
					{
					  Inf[i][j][2]=(-inf_sat[i][j]-aux1fac_sat[i][j]*Aux1M[i][j+1])*invspstep;
					}
				  }
				else if(Tc[i][j]==2)
				  {
					if(i!=n-1)
					  if(Tc[i+1][j]==0)
					{
					  Inf[i][j][0]=(inf_sat[i+1][j]+aux1fac_sat[i+1][j]*Aux1M[i][j])*invspstep;
					}
					if(i!=0)
					  if(Tc[i-1][j]==0)
					{
					  Inf[i][j][1]=(inf_sat[i-1][j]+aux1fac_sat[i-1][j]*Aux1M[i][j])*invspstep;
					}
					if(j!=m-1)
					  if(Tc[i][j+1]==0)
					{
					  Inf[i][j][2]=(inf_sat[i][j+1]+aux1fac_sat[i][j+1]*Aux1M[i][j])*invspstep;
					}
					if(j!=0)
					  if(Tc[i][j-1]==0)
					{
					  Inf[i][j][3]=(inf_sat[i][j-1]+aux1fac_sat[i][j-1]*Aux1M[i][j])*invspstep;
					}
				  }
				  
				}  
            
		for(i=0;i<n;i++)
			for(j=0;j<m;j++)
			  {		
#ifndef SATEFFLUX
					eff_sat[i][j] = eff;
#else
					
				//if(Tc[i][j]==2)
					eff_sat[i][j] = eff/(Cauxint[i][j]+SatEff);
				//else
					//eff_sat[i][j] = 0;
#endif

#ifndef SATPIN
					pPUMP_sat[i][j] = pPUMP;
#else
				//if(Tc[i][j]==2)
					pPUMP_sat[i][j] = pPUMP/(Cauxint[i][j]+SatPin);
				//else
					//pPUMP_sat[i][j] = 0;
#endif

			}
			
		for(i=0;i<n;i++)
			for(j=0;j<m;j++)
			  {
					if(Tc[i][j]==0)//wall, neighboring membrane
					  {
						if(i!=n-1)
						  if(Tc[i+1][j]==2)
						Eff[i][j][0]=eff_sat[i+1][j]*invspstep+pPUMP_sat[i+1][j]*(pinfac1*Cpin1M[i+1][j]+(1-pinfac1)*100*PinPrePat[i+1][j])*invspstep;;
						if(i!=0)
						  if(Tc[i-1][j]==2)
						Eff[i][j][1]=eff_sat[i-1][j]*invspstep+pPUMP_sat[i-1][j]*(pinfac1*Cpin1M[i-1][j]+(1-pinfac1)*100*PinPrePat[i-1][j])*invspstep;;
						if(j!=m-1)
						  if(Tc[i][j+1]==2)
						Eff[i][j][2]=eff_sat[i][j+1]*invspstep+pPUMP_sat[i][j+1]*(pinfac1*Cpin1M[i][j+1]+(1-pinfac1)*100*PinPrePat[i][j+1])*invspstep;;
						if(j!=0)
						  if(Tc[i][j-1]==2)
						Eff[i][j][3]=eff_sat[i][j-1]*invspstep+pPUMP_sat[i][j-1]*(pinfac1*Cpin1M[i][j-1]+(1-pinfac1)*100*PinPrePat[i][j-1])*invspstep;;
					  }
					else if(Tc[i][j]==2)//membrane, neighboring wall
					  {
						if(i!=n-1)
						  if(Tc[i+1][j]==0)
						Eff[i][j][0]=-eff_sat[i][j]*invspstep-pPUMP_sat[i][j]*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
						if(i!=0)
						  if(Tc[i-1][j]==0)
						Eff[i][j][1]=-eff_sat[i][j]*invspstep-pPUMP_sat[i][j]*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
						if(j!=m-1)
						  if(Tc[i][j+1]==0)
						Eff[i][j][2]=-eff_sat[i][j]*invspstep-pPUMP_sat[i][j]*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
						if(j!=0)
						  if(Tc[i][j-1]==0)
						Eff[i][j][3]=-eff_sat[i][j]*invspstep-pPUMP_sat[i][j]*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
					  }
					  
				}
 //do one direction implicit one explicit
      for(j=0;j<m;j++)
	{
	  for(i=0;i<n;i++)
	    {
	      //Compute coefficients for implicit diffusion and fluxes
	      //diffusion only
	      if(Tc[i][j]==3)
		{
		  a[i]=0;
		  c[i]=0;
		  b[i]=1;
		  rhs[i]=0;
		}
	      else
		{
		  a[i]=-DiffAuxC[i][j][1];//i-1 direction
		  c[i]=-DiffAuxC[i][j][0];//i+1 direction
		  b[i]=1-(a[i]+c[i]);//i;
		  if(Tc[i][j]==0)
		    {
		      //influx
		      b[i]+=-0.5*tstep*(Inf[i][j][0]+Inf[i][j][1]);
		      //efflux
		      a[i]+=-0.5*tstep*Eff[i][j][1];
		      c[i]+=-0.5*tstep*Eff[i][j][0];
		    }
		  else if(Tc[i][j]==2)
		    {
		      //influx
		      a[i]+=-0.5*tstep*Inf[i][j][1];
		      c[i]+=-0.5*tstep*Inf[i][j][0];
		      //efflux
		      b[i]+=-0.5*tstep*(Eff[i][j][0]+Eff[i][j][1]);
		    }
		  
		  
		  //explicit diffusion and fluxes
		  rhs[i]=Cauxint[i][j]+DiffAuxC[i][j][2]*(Cauxint[i][j+1]-Cauxint[i][j])
		    +DiffAuxC[i][j][3]*(Cauxint[i][j-1]-Cauxint[i][j]);
		  if(Tc[i][j]==0)
		    {
		      rhs[i]+=0.5*tstep*(Eff[i][j][2]*Cauxint[i][j+1]+Eff[i][j][3]*Cauxint[i][j-1]);
		      rhs[i]+=0.5*tstep*((Inf[i][j][2]+Inf[i][j][3])*Cauxint[i][j]);
		    }
		  else if(Tc[i][j]==2)
		    {
		      rhs[i]+=0.5*tstep*((Eff[i][j][2]+Eff[i][j][3])*Cauxint[i][j]);
		      rhs[i]+=0.5*tstep*(Inf[i][j][2]*Cauxint[i][j+1]+Inf[i][j][3]*Cauxint[i][j-1]);
		    }
		}
	    }
	  //Compute implicit diffusion and use this to update rhs
	  //Use new rhs to update volt2
	  dsolvetridiag(n); 
	  for(i=0;i<n;i++)
	    { 
	      if(Tc[i][j]!=3)
		{
		  func=0.5*tstep*(paux[i][j]-daux[i][j]*Cauxint[i][j]);
		  Caux2[i][j]=rhs[i]+func;
		}
	    }
	}
    }

  //bereken de gemiddelde auxine conc per cell
  for(q=0;q<totcells;q++)
    MeanCells[q]=0;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Cell[i][j]!=-1)
	  {
	    q=Cell[i][j];
	    MeanCells[q]+=Caux1[i][j];
	  }
      }
  for(q=0;q<totcells;q++)
    MeanCells[q]=MeanCells[q]*InvCellSizes[q];
  
  //final update for next timestep
  //doe dit pas na het berekenen van het 
  //gemiddelde auxin level per cell
  //hierdoor doe je genexpressie update
  //nog op basis van oude aux1, net als
  //fluxes, diffusie, decay etc.    
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]!=3)
	  Caux1[i][j]=Caux2[i][j];
      }

#endif

  
  //check symmetry auxin levels
#ifndef TROPIC
  static int counter=0;
  static int counter_2 = 0;
  //printf("time %i value %f %f %f %f\n",counter,Caux1[42][1527],Caux1[42][1526],Caux1[43][1527],Caux1[43][1526]);
  for(j=0;j<m;j++)
    for(i=0;i<=(n/2)-1;i++)
      {
	if(Tc[i][j]!=3 && counter_2 == 0)
	  {
	    if((int)(1000*(Caux1[i][j]-Caux1[n-1-i][j]))!=0)
	      {
		printf("something is wrong in auxin levels at %i %i %i (%.10f %i %i) and  %i %i %i(%.10f %i %i)\n",i,j,Cell[i][j],Caux1[i][j],(int)(10000*Caux1[i][j]),Tc[i][j],n-1-i,j,Cell[n-1-i][j],Caux1[n-1-i][j],(int)(10000*Caux1[n-1-i][j]),Tc[n-1-i][j]);
		//printf("n is %i m is %i\n",n,m);
		//printf("halfway is %i\n",xm);
		//printf("yzero %i and yend %i\n",yzero,yEND);
		//printf("nearest celltype is %i\n",Tct[i-1][j-1]);
		//printf("totalcellnr is %i\n",actcells);
		printf("counter is %i\n",counter);
		counter_2=1;
		//exit(1);
	      }
	  }
      }
  //printf("finished checking symmetry auxin levels, counter is %i\n",counter);
  counter++;
  //exit(1);
#endif
  
}



void UpdateGeneExpression(int t)
{
  int q,k;
  static double sat2plt=satplt*satplt;
  static double sat2gg=satgg*satgg;
  static double sat2pin=satpin*satpin;
  static double sat2plts=satplts*satplts;
  static double sat2yuc3=satyuc3*satyuc3;
  static double sat2gh33=satgh33*satgh33;
  static double sat2aux1=sataux1*sataux1;

  double dtARF,dtAA,dtBB,dtCC,dtDD,dtPlt,dtPlts;
  double dtPin,dtAux1;             ;
  double dtYuc3,dtGh33;
  static double tspeedupstep=tstep*speedup2;
 
  //update gene expression levels: cell based
  for(q=0;q<actcells;q++)//for(q=0;q<totcells;q++)
    {
      dtARF=0;
      dtAA=0;
      dtBB=0;
      dtCC=0;
      dtDD=0;
      dtPlt=0;
      dtPlts=0;
      dtPin=0;
      dtAux1=0;
      dtYuc3=0;
      dtGh33=0;


      //ARF depends on auxin
      double ass=0.01*0.3;
      double diss=0.001;//0.25;
      double pI=0.01;
      double dIbasal=0.0001;
      double dITIR1=0.001;
      double ARFtotal=100;
      dtARF=tspeedupstep*(diss*(ARFtotal-ARF[q])-ass*ARF[q]*(pI/(dIbasal+dITIR1*MeanCells[q])));
           
      double satA=50;
      double satB=50;
      double satC=50;
      double satD=58;
      dtAA=tspeedupstep*(maxgg*ARF[q]*ARF[q]/(ARF[q]*ARF[q]+satA*satA )-decaygg*AA[q]);
      dtBB=tspeedupstep*(maxgg*AA[q]*AA[q]/(AA[q]*AA[q]+satB*satB )-decaygg*BB[q]);
      dtCC=tspeedupstep*(maxgg*BB[q]*BB[q]/(BB[q]*BB[q]+satC*satC )-decaygg*CC[q]);
      dtDD=tspeedupstep*(maxgg*CC[q]*CC[q]/(CC[q]*CC[q]+satD*satD )-decaygg*DD[q]);

     
      //plt depends on??
      dtPlt=0;
     
             

#ifndef RESTART
      if(t<tdelay+20000)
	{
	  dtPlts=tspeedupstep*(maxplts*ARF[q]*ARF[q]/(ARF[q]*ARF[q]+sat2plts)-decayplts*Plts[q]);
	  Pltsexpr[q]=100*ARF[q]*ARF[q]/(ARF[q]*ARF[q]+sat2plts);
	}
      else
#endif
	{
	  double temp;
	  temp=(DD[q]*DD[q]/(DD[q]*DD[q]+sat2plts));
	  dtPlts=tspeedupstep*(maxplts*temp-decayplts*Plts[q]);
	  Pltsexpr[q]=100*(temp);		   
	}
     
     
#ifdef PINREGULATED 
      if(Tct[(int)(0.5*(CellMinI[q]+CellMaxI[q]))][(int)(0.5*(CellMinJ[q]+CellMaxJ[q]))]<=7) 
		dtPin=tspeedupstep*(0.1*maxpin+0.9*maxpin*(MeanCells[q]*MeanCells[q]/(MeanCells[q]*MeanCells[q]+km_pin2*km_pin2))-decaypin*Pin[q]);
		//dtPin=tspeedupstep*(0.1*maxpin+0.9*maxpin*(ARF[q]*ARF[q]/(ARF[q]*ARF[q]+3*3))-decaypin*Pin[q]);
	  else if(Tct[(int)(0.5*(CellMinI[q]+CellMaxI[q]))][(int)(0.5*(CellMinJ[q]+CellMaxJ[q]))]>7 && Tct[(int)(0.5*(CellMinI[q]+CellMaxI[q]))][(int)(0.5*(CellMinJ[q]+CellMaxJ[q]))]<=11)
		//dtPin=tspeedupstep*(0.1*maxpin+0.9*maxpin*(ARF[q]*ARF[q]/(ARF[q]*ARF[q]+2*2))-decaypin*Pin[q]);
		dtPin=tspeedupstep*(0.3*maxpin+0.7*maxpin*(MeanCells[q]*MeanCells[q]/(MeanCells[q]*MeanCells[q]+km_pin1*km_pin1))-decaypin*Pin[q]);
      else
		dtPin=0;
#endif


      //Pin,Yuc3,Gh33,Lax2 all depend on pltslow
#ifdef YUC3GH33REGULATED
      dtYuc3=tspeedupstep*(maxyuc3*Plts[q]*Plts[q]/(Plts[q]*Plts[q]+sat2yuc3)-decayyuc3*Yuc3[q]);
      dtGh33=tspeedupstep*(maxgh33*sat2gh33/(Plts[q]*Plts[q]+sat2gh33)-decaygh33*Gh33[q]);
#endif


#ifdef AUX1REGULATED
		
      dtAux1=tspeedupstep*(maxaux1*(MeanCells[q]*MeanCells[q]/(MeanCells[q]*MeanCells[q]+20*20))-decayaux1*Aux1[q]); 
#endif


    

   ARF[q]+=dtARF;
   AA[q]+=dtAA;
   BB[q]+=dtBB;
   CC[q]+=dtCC;
   DD[q]+=dtDD;
   Plt[q]+=dtPlt;
   Plts[q]+=dtPlts;
   Pin[q]+=dtPin;
   Aux1[q]+=dtAux1;
   Yuc3[q]+=dtYuc3;
   Gh33[q]+=dtGh33;

}

#ifdef PLASMODESMATAOPEN
  int i,j,ii,jj;
  double pdflux=4*0.01/(double)xstep;
  //initialise PLTs on grid
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Cell[i][j]!=-1)//cell there
	  {
	    q=Cell[i][j];
	    Pltsgrid[i][j]=Plts[q];
	    Pltsgrid2[i][j]=Plts[q];
	  }
	else
	  {
	    Pltsgrid[i][j]=0;
	    Pltsgrid2[i][j]=0;
	  }
      }
 
  //PLTs fluxes between cells through plasmodesmata
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==2)//mebrane, so point of possible exchange
	  {
	    if(Tc[i-1][j]==0)//wall to left, so left membrane
	      {
		ii=i-1;
		while(ii>0 && (i-ii)<4)
		  {
		    ii--;
		    if(Tc[ii][j]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[ii][j]-Pltsgrid[i][j]);
			break;
		      }
		  }
	      }
	    if(Tc[i+1][j]==0)//wall to right, so right membrane
	      {
		ii=i+1;
		while(ii<n && (ii-i)<4)
		  {
		    ii++;
		    if(Tc[ii][j]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[ii][j]-Pltsgrid[i][j]);
			break;
		      }
		  }
	      }
	    if(Tc[i][j-1]==0)//wall above, so apical membrane ???????
	      {
		jj=j-1;
		while(jj>0 && (j-jj)<4)
		  {
		    jj--;
		    if(Tc[i][jj]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[i][jj]-Pltsgrid[i][j]);
			break;
		      }		 
		  }
	      }
	    if(Tc[i][j+1]==0)//wall below, so basal membrane ?????????
	      {
		jj=j+1;
		while(jj<m && (jj-j)<4)
		  {
		    jj++;
		    if(Tc[i][jj]==2)//neighbouring membrane found
		      {
			Pltsgrid2[i][j]+=pdflux*(Pltsgrid[i][jj]-Pltsgrid[i][j]);
			break;
		      }		 
		  }
	      }
	  }
      }
#ifdef PLTLEAKAGETOMAINROOT
  double Pltslevelshoot=0;
  double pdfluxshoot=pdflux;
  for(i=0;i<n;i++)
    for(j=m-1;j>=0;j--)
      {
	if(Cell[i][j]!=-1)//topmost part of topmost cell found
	  {
	    Pltsgrid2[i][j]+=pdfluxshoot*(Pltslevelshoot-Pltsgrid[i][j]);
	    j=-100;//so break from second for loop
	  }
      }
#endif
  
    
  //transform back from grid to cellular
  //assumption that within cell PLTs differences
  //even out quickly through diffusion
  for(q=0;q<totcells;q++)
    Plts[q]=0;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Cell[i][j]!=-1)//cell there
	  {
	    q=Cell[i][j];
	    Plts[q]+=Pltsgrid2[i][j];
	  }
      }
  for(q=0;q<actcells;q++)
    Plts[q]*=(double)InvCellSizes[q];
 
#endif
}



void UpdateMembraneProteinsAndFluxes(int time)
{
  int i,j,k,q,ii;

  static int counter=0;
  FILE *f;
  //int ioutside=(int)(N/2);
  //int iinside=(int)(3.5*N+7*z1);
  //int jtopoutside;
 // int jtopinside;
 // double fluxoutside;
 // double fluxinside;
  double invspstep=1.0/(double)(xstep);


 //SetUpTissueLayout();
  //SetUpMembraneTypes();
  
//In principe update je de PinPrePat, membranes, membrane types 
  //en celltypes tijdens growth en zou je hier alleen de cortex flip 
  //en evt andere zone dependent changes moeten doen
  //for(i=0;i<n;i++)
    //for(j=0;j<m;j++)
      //{
	//if(Cell[i][j]!=-1)//cell
	  //{
	    //if(Tc[i][j]==2)//membrane
	      //{
		//q=Cell[i][j];
		//if(CellZonationState[q]>30)//not in MZ, so went to EZ/DZ
		  //{
		    //if(Tct[i][j]==6)
		      //{
			//if(Tmt[i][j]==2)//basal membrane
			  //PinPrePat[i][j]=0;
			//else if(Tmt[i][j]==1)//apical membrane
			  //PinPrePat[i][j]=1;
		      //}
		  //}
	      //}
	  //}
      //} 
 
     
      
 // int ii;
  for(i=xm;i>=0;i--)
    for(j=0;j<m;j++)
    {
		ii=n-1-i;
		PinPrePat[i][j]=0;
		PinPrePat[ii][j]=0;
		Aux1PrePat[i][j]=0;
		Aux1PrePat[ii][j]=0;
    }

#ifdef POLAR
  for(i=xm;i>=0;i--)
    for(j=0;j<m;j++)
      {
	ii=n-1-i;
	q=Cell[i][j];
#ifdef APOLARMERISTEM
if(Tc[i][j]==2)//membrane
	  {
	    if(Tct[i][j]<=4) //lateral root cap
	    {
			if(Tmt[i][j]==1)//up
			{
			PinPrePat[i][j]=0.5;
			PinPrePat[ii][j]=0.5;
			}
			else if(Tmt[i][j]==3)//in
			{
			PinPrePat[i][j]=0.1;//.1
			PinPrePat[ii][j]=0.1;//.1
			}
	    }

	    else if(Tct[i][j]==5)//epidermis
	      {

			if(Tmt[i][j]==1)//up
			{
				PinPrePat[i][j]=1;
				PinPrePat[ii][j]=1;
			}
			else if(Tmt[i][j]==3)//in
	        {
				  if(j<yR)
				  {
					PinPrePat[i][j]=0.5;
					PinPrePat[ii][j]=0.5;
				  }
				  else if(CellZonationState==0)
				  {
					  PinPrePat[i][j]=0.25;
					PinPrePat[ii][j]=0.25;
				  }  
				  else
				  {
					PinPrePat[i][j]=0.1;
					PinPrePat[ii][j]=0.1;
				  }
			 }
		  }
		else if(Tct[i][j]==6)//cortex
	      {
		
			if(Tmt[i][j]==1 && CellZonationState[q]>30) //j>m/3.5) //j>YR)//up
			  {
		   
				PinPrePat[i][j]=1;
				PinPrePat[ii][j]=1;
			  }
			else if(Tmt[i][j]==2 && CellZonationState[q]<=30) //j<m/4) //j<YR)//down
			  {
				
				PinPrePat[i][j]=1;
				PinPrePat[ii][j]=1;
			  }
			else if(Tmt[i][j]==3)//in
		      {
				  if(j<yR)
				  {
					PinPrePat[i][j]=0.5;
					PinPrePat[ii][j]=0.5;
				  }
				  else if(CellZonationState==0)
				  {
					PinPrePat[i][j]=0.25;
					PinPrePat[ii][j]=0.25;
				  }	  
				  else
				  {
					PinPrePat[i][j]=0.1;
					PinPrePat[ii][j]=0.1;
				  }
			 }
		  }
	    else if(Tct[i][j]==7)//endodermis
	      {
			if(Tmt[i][j]==2)//down
			  {
				PinPrePat[i][j]=1;
				PinPrePat[ii][j]=1;
			  }
			else if(Tmt[i][j]==1 && j<yR) //Up
			{
				PinPrePat[i][j]=1;
				PinPrePat[ii][j]=1;
			}
			else if(Tmt[i][j]==3)//in
		    {
				if(j<yR)
				{
					PinPrePat[i][j]=1;
					PinPrePat[ii][j]=1;
				}
				else if(CellZonationState[q]==0)
				{
					PinPrePat[i][j]=0.7;
					PinPrePat[ii][j]=0.7;
				}
				else 
				{
					PinPrePat[i][j]=0.2;
					PinPrePat[ii][j]=0.2;
				}
			
		    }
			else if(Tmt[i][j]==4)//in
			{
				if(j<yR)
				{
					PinPrePat[i][j]=1;
					PinPrePat[ii][j]=1;
				}
				else if(CellZonationState[q]==0)
				{
					PinPrePat[i][j]=0.7;
					PinPrePat[ii][j]=0.7;
				}
				else if(CellZonationState[q]==30)
				{
					PinPrePat[i][j]=0.1;
					PinPrePat[ii][j]=0.1;
				}			
			}
	      }
	    else if(Tct[i][j]==8)//pericycle
	      {
			if(j>(int)(arad3*i+brad3))
			  {
				if(Tmt[i][j]==2)//down
				  {
					PinPrePat[i][j]=1;
					PinPrePat[ii][j]=1;
				  }
				else if(Tmt[i][j]==3)//in
				  {
				  if(j<yR)
					{
						PinPrePat[i][j]=1;
						PinPrePat[ii][j]=1;
					}
				  else if(CellZonationState[q]==0)
					{
						PinPrePat[i][j]=0.7;
						PinPrePat[ii][j]=0.7;
					}
				  else 
					{
						PinPrePat[i][j]=0.2;
						PinPrePat[ii][j]=0.2;
					}
				  }
				else if(Tmt[i][j]==4)//out
				 {
				  if(j<yR)
					{
						PinPrePat[i][j]=1;
						PinPrePat[ii][j]=1;
					}
				  else if(CellZonationState[q]==0)
					{
						PinPrePat[i][j]=0.7;
						PinPrePat[ii][j]=0.7;
					}
				  else 
					{
						PinPrePat[i][j]=0.1;
						PinPrePat[ii][j]=0.1;
					}
				 }
			   }
			else
			  {
				if(Tmt[i][j]==4 || Tmt[i][j]==3 || Tmt[i][j]==2 || Tmt[i][j]==1)//(down is defined as out here, due to circularity)
				  {
					PinPrePat[i][j]=1;
					PinPrePat[ii][j]=1;
				  }
			  }
			}
	    else if(Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11)//vasculature
	      {
		    if(Tmt[i][j]==2)//down
				{ 
					PinPrePat[i][j]=1;
					PinPrePat[ii][j]=1;
				}
			else if(Tmt[i][j]==1 && j<yR) //Up
				{
					PinPrePat[i][j]=1;
					PinPrePat[ii][j]=1;
				}
			else if(Tmt[i][j]==3)//inward membrane
				{
					if(j<yR)
					{
						PinPrePat[i][j]=1;
						PinPrePat[ii][j]=1;
					}
					else if(CellZonationState[q]==0)
					{
						PinPrePat[i][j]=0.7;
						PinPrePat[ii][j]=0.7;
					}
					else if(CellZonationState[q]==30)
					{
						PinPrePat[i][j]=0.2;
						PinPrePat[ii][j]=0.2;
					}
				}
			else if(Tmt[i][j]==4)//out 
			  {
					if(j<yR)
					{
						PinPrePat[i][j]=1;
						PinPrePat[ii][j]=1;
					}
					else if(CellZonationState[q]==0)
					{
						PinPrePat[i][j]=0.7;
						PinPrePat[ii][j]=0.7;
					}
					else if(CellZonationState[q]==30)
					{
						PinPrePat[i][j]=0.1;
						PinPrePat[ii][j]=0.1;
					}
				}
			}
	     else if(Tct[i][j]==12)//QC
	      {
			if(Tmt[i][j]==4)//(down is defined as out here, due to circularity)
			  {
				PinPrePat[i][j]=1;
				PinPrePat[ii][j]=1;
			  }
	      }
	    else if(Tct[i][j]==13)//columella bottom layer
	      {
			if(Tmt[i][j]<4)
			  {
				PinPrePat[i][j]=0.5;
				PinPrePat[ii][j]=0.5;
			  }
	      }
	    else if(Tct[i][j]>13)//columella
	       {
				PinPrePat[i][j]=1;
				PinPrePat[ii][j]=1;
	       }
	   
	  }
#endif	  

#ifdef POLARMERISTEM
	if(Tc[i][j]==2)//membrane
	  {
	    if(Tct[i][j]<=4)//lateral root cap
	      {

		if(Tmt[i][j]==1)// && Tc[i][j+2]!=3)//up
		  {
		    PinPrePat[i][j]=0.5;
		    PinPrePat[ii][j]=0.5;
		  }
		else if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
	      }
	    else if(Tct[i][j]==5)//epidermis
	      {

		if(Tmt[i][j]==1)//up
		  {
		    //if(Tc[i][j+2]!=3)
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		      /*else
		      {
			PinPrePat[i][j]=0.1;
			PinPrePat[ii][j]=0.1;
			}*/
		  }
		else if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
	      }
	    else if(Tct[i][j]==6)//cortex
	      {
		
		if(Tmt[i][j]==1 && CellZonationState[Cell[i][j]]>30)//up
		  {
		    //if(Tc[i][j+2]!=3)
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		      /*else
		      {
			PinPrePat[i][j]=0.1;
			PinPrePat[ii][j]=0.1;
			}*/
		  }
		else if(Tmt[i][j]==2 && CellZonationState[Cell[i][j]]<=30)//down
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		else if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
	      }
	    else if(Tct[i][j]==7)//endodermis
	      {
		if(Tmt[i][j]==2)//down
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		else if(Tmt[i][j]==3)//in
		  {
		    PinPrePat[i][j]=0.35;
		    PinPrePat[ii][j]=0.35;
		  }
	      }
	    else if(Tct[i][j]==8)//pericycle
	      {
		if(j>(int)(arad4*i+brad4))
		  {
		    if(Tmt[i][j]==2)//down
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		   else if(Tmt[i][j]==3)//in, EXTRA????!!!
		      {
			PinPrePat[i][j]=0.1;
			PinPrePat[ii][j]=0.1;
		      }
		  }
		else
		  {
		    if(Tmt[i][j]==4)//(down is defined as out here, due to circularity)
		      {
			PinPrePat[i][j]=1;
			PinPrePat[ii][j]=1;
		      }
		  }
	      }
	    else if(Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11)//vasculature
	      {
		if(Tmt[i][j]==2)//down
		  {
		   
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
		else if(Tmt[i][j]==4)//out //EXTRA, TO ENHANCE AUXIN IN PERICYCLE AND PROTOXYLEM
		  {
		    PinPrePat[i][j]=0.1;
		    PinPrePat[ii][j]=0.1;
		  }
	      }
	    else if(Tct[i][j]==12)//QC
	      {
		if(Tmt[i][j]==4)//(down is defined as out here, due to circularity)
		  {
		    PinPrePat[i][j]=1;
		    PinPrePat[ii][j]=1;
		  }
	      }
	    else if(Tct[i][j]==13)//columella bottom layer
	      {
		if(Tmt[i][j]<4)
		  {
		    PinPrePat[i][j]=0.5;
		    PinPrePat[ii][j]=0.5;
		  }
	      }
	    else if(Tct[i][j]>13)//>12)//columella
	      {
		PinPrePat[i][j]=1;
		PinPrePat[ii][j]=1;
	      }
	   
	  }
#endif
      }
#endif


for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
#ifdef AUX1INEPI
	if((Tct[i][j]!=6 && Tct[i][j]!=7) && (Tct[i][j]!=12 && Tct[i][j]!=18))//if((Tct[i][j]!=6 && Tct[i][j]!=7) && (Tct[i][j]!=10 && Tct[i][j]!=16))
	  {
	    
			Aux1PrePat[i][j]=1;
		//if((Tct[i][j]==8 || Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11) && j<yMZEZ)
			//Aux1PrePat[i][j]=2;
		
	   if(Tct[i][j]==5  && j<yMZEZ)//((Tct[i][j]==5 ||Tct[i][j]==6) && j<yMZEZ)
	      Aux1PrePat[i][j]=0.3;//0.8;
#ifdef AUX1LRCextra
		else if(Tct[i][j]==1 || ((Tct[i][j]==8 || Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11) && j<yMZEZ-6*HMZ))
			Aux1PrePat[i][j]=3;
#endif
	  }
	else
#endif
	  Aux1PrePat[i][j]=0;
      }

#ifdef CYTOKININ

double ttime=time%(3*3600);
double phase;


	if(ttime<(1*3600))//2.5
		phase=-1;
	else
		phase=1;
//dit geeft een cycle van 5 uur met 2.5 uur down 2.5 uur up


double factor_AUX1;
double factor_pAux;


	if(phase==-1)
		{
		   factor_AUX1=0.8;
		   factor_pAux=1;
		}
	else
		{
		  factor_AUX1=1;
		  factor_pAux=1;//10
		}

#endif

 //update membrane protein concentration based on new prepatterns and gene expression levels
  

  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Cell[i][j]!=-1)//grid point inside cell
	  {
	    q=Cell[i][j];
	    //Cpin1M[i][j]=Pin[Cell[i][j]]*PinPrePat[i][j];
    
#ifdef AUXINREGULATESPIN

	//Use [auxin] in cell-wall for feedback 
	ii=n-1-i;  
	   if(Tct[i][j]<=5  && Tct[i][j]!=0) //Feedabck in epidermis and lateral root cap
	  
	   {
		   if(Tmt[i][j]==1)//up
		   {
				if(Tc[i][j]==2)
				{
					if(Tc[i][j+1]==0)
					Cpin1M[i][j]=Pin[q]*PinPrePat[i][j]*(.1+1.5*(((Caux1[i][j+1])*(Caux1[i][j+1]))/((Caux1[i][j+1])*(Caux1[i][j+1])+10*10))); //Km=50
				}
			}
			
			else if(Tmt[i][j]==3) //in
			{
				if(i<n/2)
				{
					if(Tc[i+1][j]==0)
					Cpin1M[i][j]=Pin[q]*PinPrePat[i][j]*(.1+1.5*(((Caux1[i+1][j])*(Caux1[i+1][j]))/((Caux1[i+1][j])*(Caux1[i+1][j])+10*10)));
				}									
				else 
				{
					if(Tc[i-1][j]==0)
					Cpin1M[i][j]=Pin[q]*PinPrePat[i][j]*(.1+1.5*(((Caux1[i-1][j])*(Caux1[i-1][j]))/((Caux1[i-1][j])*(Caux1[i-1][j])+10*10)));
				}
			}
			else
					Cpin1M[i][j]=Pin[q]*PinPrePat[i][j];
		}
	
	   else
	
#endif	
		 Cpin1M[i][j]=Pin[q]*PinPrePat[i][j];
	 
#ifdef CYTOKININ
		if((Tct[i][j]==5 || Tct[i][j]==1 || Tct[i][j]==8 || Tct[i][j]==9) && (j>=yOZs && j<=yOZe))//((Tct[i][j]==8 || Tct[i][j]==9) && j<yMZEZ) || Tct[i][j]==15)//(Tct[i][j]==1 && j>yQC+13*HMZ+11) || 
			Aux1M[i][j]=Aux1[Cell[i][j]]*Aux1PrePat[i][j]*factor_AUX1;
		else
#endif
			Aux1M[i][j]=Aux1[Cell[i][j]]*Aux1PrePat[i][j];	  


	
	   

	  }
	else
	  {
	    Cpin1M[i][j]=0;
	    Aux1M[i][j]=0;
	  }   	
      }
 

  //update auxin fluxes and metabolism based on new tissue architecture and gene expression levels
  //auxin prod, decay, diff, flux rates:
  //direction 0 i+1;
  //direction 1 i-1;
  //direction 2 j+1;
  //direction 3 j-1;
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	daux[i][j]=0;//default no decay
	paux[i][j]=0;//default no production
	for(k=0;k<4;k++)
	  {	  
	    Inf[i][j][k]=0;
	    Eff[i][j][k]=0;
	    DiffAuxC[i][j][k]=0;
	  }
      }
 

  //daux depends on Gh33 gene expression
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	q=Cell[i][j];
	if(q!=-1)
	  daux[i][j]=(1-gh33fac)*dAux+gh33fac*0.01*Gh33[q]*dAux;
      }
  //paux depends on Yuc3 expression
  //double volfactor;
  //for(i=0;i<n;i++)
    //for(j=0;j<m;j++)
      //{
	//q=Cell[i][j];
	//if(q!=-1)
	  //{
	    //volfactor=1;//(4/(xstep*xstep))*32*InvCellSizes[Cell[i][j]];
	    //paux[i][j]=volfactor*((1-yuc3fac)*pAux+yuc3fac*0.01*Yuc3[q]*pAux);
	  //}
      //}
double volfactor;



  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
	{
	q=Cell[i][j];
	if(q!=-1)
		{
	ii=n-1-i;		
#ifdef LOWAUXIN_FROMSHOOT
			if((Tct[i][j]==12) || (Tct[i][j]==17) || (j<(int)(aPERI*abs(pow((i-xm),powPERI))+bPERI)+sizevasc && (Tct[i][j]==9 || Tct[i][j]==10 || Tct[i][j]==11))  || (j<(int)(arad3*i+brad3) && j<(int)(arad3*ii+brad3) && (Tct[i][j]==5 || Tct[i][j]==6 || Tct[i][j]==7 || Tct[i][j]==8)))//Collumella & surrounding cells
			{

				//	
				volfactor=50;
//#ifdef CYTOKININ
				//volfactor=50*factor_pAux;
//#endif
				//paux[i][j]=volfactor*((1-yuc3fac)*pAux+yuc3fac*0.01*Yuc3[q]*pAux);
				//paux[ii][j]=volfactor*((1-yuc3fac)*pAux+yuc3fac*0.01*Yuc3[q]*pAux);
			}
			else if(Tct[i][j]==1 || Tct[i][j]==2) // || Tct[i][j]==3 || Tct[i][j]==4) // || Tct[i][j]==2 || Tct[i][j]==3 || Tct[i][j]==4)//Lateral Root Cap
			{
	
				volfactor=10;
				////paux[i][j]=volfactor*((1-yuc3fac)*pAux+yuc3fac*0.01*Yuc3[q]*pAux);
				////paux[ii][j]=volfactor*((1-yuc3fac)*pAux+yuc3fac*0.01*Yuc3[q]*pAux);
			}
			else 
#endif
				volfactor=1;
		
				
		paux[i][j]=volfactor*((1-yuc3fac)*pAux+yuc3fac*0.01*Yuc3[q]*pAux);
		//paux[ii][j]=volfactor*((1-yuc3fac)*pAux+yuc3fac*0.01*Yuc3[q]*pAux);
      }

}


#ifdef AUXINAPPLICATION
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(i==0 || i==n-1 || j==0)
	  paux[i][j]+=1.00/xstep;
      }
#endif


#ifndef SATURATION
  //Set Inf from Lax3,Lax2,Aux1
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)
	  {
	    //nearest cell:
	    if(i!=0)
	      if(Tc[i-1][j]==2)
		{
		  Inf[i][j][1]=(-inf-aux1fac*Aux1M[i-1][j])*invspstep;
		}
	    if(i!=n-1)	
	      if(Tc[i+1][j]==2)
		{
		  Inf[i][j][0]=(-inf-aux1fac*Aux1M[i+1][j])*invspstep;
		}
	    if(j!=0)
	      if(Tc[i][j-1]==2)
		{
		  Inf[i][j][3]=(-inf-aux1fac*Aux1M[i][j-1])*invspstep;
		}
	    if(j!=m-1)
	      if(Tc[i][j+1]==2)
		{
		  Inf[i][j][2]=(-inf-aux1fac*Aux1M[i][j+1])*invspstep;
		}
	  }
	else if(Tc[i][j]==2)
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		{
		  Inf[i][j][0]=(inf+aux1fac*Aux1M[i][j])*invspstep;
		}
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		{
		  Inf[i][j][1]=(inf+aux1fac*Aux1M[i][j])*invspstep;
		}
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		{
		  Inf[i][j][2]=(inf+aux1fac*Aux1M[i][j])*invspstep;
		}
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		{
		  Inf[i][j][3]=(inf+aux1fac*Aux1M[i][j])*invspstep;
		}
	  }
      }
  //Set Eff from Tc
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)//wall, neighboring membrane
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==2)
		Eff[i][j][0]=eff*invspstep+pPUMP*(pinfac1*Cpin1M[i+1][j]+(1-pinfac1)*100*PinPrePat[i+1][j])*invspstep;;
	    if(i!=0)
	      if(Tc[i-1][j]==2)
		Eff[i][j][1]=eff*invspstep+pPUMP*(pinfac1*Cpin1M[i-1][j]+(1-pinfac1)*100*PinPrePat[i-1][j])*invspstep;;
	    if(j!=m-1)
	      if(Tc[i][j+1]==2)
		Eff[i][j][2]=eff*invspstep+pPUMP*(pinfac1*Cpin1M[i][j+1]+(1-pinfac1)*100*PinPrePat[i][j+1])*invspstep;;
	    if(j!=0)
	      if(Tc[i][j-1]==2)
		Eff[i][j][3]=eff*invspstep+pPUMP*(pinfac1*Cpin1M[i][j-1]+(1-pinfac1)*100*PinPrePat[i][j-1])*invspstep;;
	  }
	else if(Tc[i][j]==2)//membrane, neighboring wall
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		Eff[i][j][0]=-eff*invspstep-pPUMP*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		Eff[i][j][1]=-eff*invspstep-pPUMP*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		Eff[i][j][2]=-eff*invspstep-pPUMP*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		Eff[i][j][3]=-eff*invspstep-pPUMP*(pinfac1*Cpin1M[i][j]+(1-pinfac1)*100*PinPrePat[i][j])*invspstep;
	  }
      }
#endif

  //Set DiffAuxC from Tc
  double DauxCC=0.5*DauxC*tstep/(xstep*xstep);
  double DauxWW=0.5*DauxW*tstep/(xstep*xstep);
  for(i=0;i<n;i++)
    for(j=0;j<m;j++)
      {
	if(Tc[i][j]==0)//wall grid point: only diffuse towards other wall grid point
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==0)
		DiffAuxC[i][j][0]=DauxWW;
	    if(i!=0)
	      if(Tc[i-1][j]==0)
		DiffAuxC[i][j][1]=DauxWW;
	    if(j!=m-1)
	      if(Tc[i][j+1]==0)
		DiffAuxC[i][j][2]=DauxWW;
	    if(j!=0)
	      if(Tc[i][j-1]==0)
		DiffAuxC[i][j][3]=DauxWW;
	  }
	else if(Tc[i][j]==1 || Tc[i][j]==2)//cell grid point: only diffuse towards other cell grid point
	  {
	    if(i!=n-1)
	      if(Tc[i+1][j]==1 || Tc[i+1][j]==2)
		DiffAuxC[i][j][0]=DauxCC;
	    if(i!=0)
	      if(Tc[i-1][j]==1 || Tc[i-1][j]==2)
		DiffAuxC[i][j][1]=DauxCC;
	    if(j!=m-1)
	      if(Tc[i][j+1]==1 || Tc[i][j+1]==2)
		DiffAuxC[i][j][2]=DauxCC;
	    if(j!=0)
	      if(Tc[i][j-1]==1 || Tc[i][j-1]==2)
		DiffAuxC[i][j][3]=DauxCC;
	  }
      }

  //Now we need to adjust for getting a normal influx/efflux relation between root and shoot
#ifdef NORMALSHOOTROOTCONNECTION
  double auxinvirtlayer=0.8*0.5;//1;//0.5;//10;//0.5
#endif
#ifdef SHOOTROOTCUT
  double auxinvirtlayer=0.8*0.5*exp(-0.003*time);//0;
#endif
#ifdef LOWAUXIN_FROMSHOOT 
  double auxinvirtlayer=0.8*0.5*0.66;
#endif
  int jbot;
  int jstore;

  for(i=0;i<n;i++)
    {
      for(j=m-1;j>=0;j--)
	{
	 
	  if(Tc[i][j]==0 && Tc[i][j-1]==2)//wall above cell
	    {
	      if(Tct[i][j-1]==5 || Tct[i][j-1]==6 || Tct[i][j-1]==7)//efflux to shoot from epi/cort/endo
		{		  
		  jbot=CellMinJ[Cell[i][j-1]]-1;
		  daux[i][j]=-0.1*Inf[i][jbot][2];
		}
	      else if(Tct[i][j-1]==8 || Tct[i][j-1]==9)//influx from shoot to pericycle and vasculature
		{
		  jbot=CellMinJ[Cell[i][j-1]]-1;
		  paux[i][j]=(Eff[i][jbot][2])*auxinvirtlayer;
		}
	      break;
	    }
	}
    }
}

